#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <fitsio.h>

/* Continuum errors */
#define ERROR_RRC    0.01
#define ERROR_TWOPH  0.01
#define ERROR_BREMS  0.01
#define ERROR_PSEUDO 0.01
#define ERROR_FINAL  0.01
#define ERG_KEV 1.60219e-9

#define MIN_EMIS 1.e-38   /* in photons cm^3 s^-1 bin^-1 */

void fits_spec_write_cocoHDU(int N_Z, int list_Z[], int list_rmJ[], 
			     fitsfile *fptr, 
			     double T, double Ne, double tot_emission, 
			     double **E_real, double **E_pseudo, 
			     double **real_cont, double **pseudo_cont,
			     int *Ncomp_real, int *Ncomp_pseudo);
void compress_photon_spectrum(double *E, double *flux, int *Nbin,double error);

/****************************************************************************/
/* This routine takes an open fits file pointer (opened using               */
/* fits_init_specout) and writes line emission data.                        */
/* Inputs:                                                                  */
/* fptr       : fits file pointer for compressed continuum file             */
/* N_Z        : number of element & ion pairs                               */
/* list_Z     : Array of elemental Z values                                 */
/* list_rmJ   : Array of ion values, roman numeral value (I = 1, etc)       */
/* T          : Temperature of plasma (K)                                   */
/* n_e        : Electron density in plasma (cm^-3)                          */
/* nbin       : Total number of bins in the default binning                 */
/* E_low      : Array of lower edge of energy bins, in keV                  */
/* E_high     : Array of higher edge of energy bins, in keV.                */
/* pseu_arr   : Array of pseudocontinuum, in [Z,iBin] = iZ + N_Z*iBin       */
/* cont_arr   : Array of real continuum with [Z,iBin] = iZ + N_Z*iBin       */
/* Note: Units on these four arrays are photon cm^3 / s / bin               */
/****************************************************************************/

void fits_spec_write_coco(fitsfile *fptr, int N_Z, int list_Z[], 
			  int list_rmJ[], double T, double Ne, int nbin,  
			  double E_low[], double E_high[], double pseu_arr[], 
			  double cont_arr[]) {

  int iZ;
  int iBin, iPos;
  double tot_emission, E_cnvt;

  /* Continuum related */
  int *Ncomp_real, *Ncomp_pseudo;
  double **E_real, **E_pseudo;
  double **real_cont, **pseudo_cont;


  E_real   = (double **) malloc(N_Z*sizeof(double *));
  E_pseudo = (double **) malloc(N_Z*sizeof(double *));

  real_cont   = (double **) malloc(N_Z*sizeof(double));
  pseudo_cont = (double **) malloc(N_Z*sizeof(double));

  Ncomp_real   = (int *) malloc(N_Z*sizeof(int));
  Ncomp_pseudo = (int *) malloc(N_Z*sizeof(int));

  for (iZ=0;iZ<N_Z;iZ++) {
    Ncomp_real[iZ]  = nbin;
    Ncomp_pseudo[iZ]= nbin;
    E_real[iZ]      = (double *) malloc((nbin+1)*sizeof(double));
    E_pseudo[iZ]    = (double *) malloc((nbin+1)*sizeof(double));
    real_cont[iZ]   = (double *) malloc((nbin+1)*sizeof(double));
    pseudo_cont[iZ] = (double *) malloc((nbin+1)*sizeof(double));
    for (iBin=0;iBin<nbin+1;iBin++) {
      real_cont[iZ][iBin]  = 0.0;
      pseudo_cont[iZ][iBin]= 0.0;
    }
  }
  tot_emission = 0.0;
  /* tot_emission: total continuum emission power, in ergs cm^3 / s  */
  /* printf("NZ= %d, nbin = %d\n",N_Z,nbin); */

  for (iZ=0;iZ<N_Z;iZ++) {
  
    (E_real[iZ])[0]  = E_low[iZ + 0*N_Z]; /* in keV */
    (E_pseudo[iZ])[0]= E_low[iZ + 0*N_Z]; /* in keV */

    for (iBin=0;iBin<nbin;iBin++) {  /* First sum everything up */

      iPos = iZ + iBin*N_Z;
      (E_real[iZ])[iBin+1]   = E_high[iPos];  
      (E_pseudo[iZ])[iBin+1] = E_high[iPos];  
      (real_cont[iZ])[iBin]  = cont_arr[iPos];
      (pseudo_cont[iZ])[iBin]= pseu_arr[iPos];

      E_cnvt = ((E_low[iPos] + E_high[iPos])/2.0)*ERG_KEV;
      tot_emission += ((real_cont[iZ])[iBin] + (pseudo_cont[iZ])[iBin])*E_cnvt;

      if ((real_cont[iZ])[iBin] < MIN_EMIS) (real_cont[iZ])[iBin]=0.0;
      if ((pseudo_cont[iZ])[iBin]<MIN_EMIS) (pseudo_cont[iZ])[iBin]=0.0;
    }

    compress_photon_spectrum(E_real[iZ], real_cont[iZ], &(Ncomp_real[iZ]),
			     ERROR_FINAL);
    compress_photon_spectrum(E_pseudo[iZ], pseudo_cont[iZ],&(Ncomp_pseudo[iZ]),
			     ERROR_FINAL);
  }
  
  /* Now we write the compressed continuum */
    
  fits_spec_write_cocoHDU(N_Z, list_Z, list_rmJ, fptr, T, Ne, tot_emission, 
			  E_real, E_pseudo, real_cont, pseudo_cont, 
			  Ncomp_real, Ncomp_pseudo);
  
  /* Now, clear out some memory. */
  
  free(Ncomp_real);
  free(Ncomp_pseudo);
  for (iZ=0;iZ<N_Z;iZ++) {
    free(E_real[iZ]);
    free(E_pseudo[iZ]);
    free(real_cont[iZ]);
    free(pseudo_cont[iZ]);
  }
  free(E_real);
  free(E_pseudo);
  free(real_cont);
  free(pseudo_cont);
}
