#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <fitsio.h>

#define MAXSTRLEN 128
#define ERG_KEV 1.60219e-9

void fits_printerror(char *process, int status);
/****************************************************************************/
/* This routine takes an open fits file pointer (opened using               */
/* fits_init_specout) and writes continuum emission data.                   */
/* Inputs:                                                                  */
/* fptr       : fits file pointer for line file                             */
/* T          : Temperature of plasma (K)                                   */
/* n_e        : Electron density in plasma (cm^-3)                          */
/* nbin       : Total number of bins                                        */
/* Ebin_low   : Array of lower edges of energy bins, in keV                 */
/* Ebin_high  : Array of higher edges of energy bins, in keV                */
/* pseudo     : Array of pseudo continuum values, in phot cm^3 / s          */
/* pseudo_err : Errors in above, or NULL if no errors are available         */
/* brems      : Array of bremsstrahlung values, in phot cm^3 / s            */
/* brems_err  : Errors in the bremsstahlung emission, or NULL if not avail. */
/* twoph      : Array of two photon emission values, in phot cm^3 / s       */
/* twoph_err  : Errors in the two photon emission, or NULL if not available */
/* radrec     : Array of radiative recombination emission in phot cm^3 / s  */
/* radrec_err : Errors in the RRC emission, or NULL if not available        */
/****************************************************************************/

void fits_spec_write_continuum(fitsfile *fptr, double T, double n_e,
			       int nbin, double Ebin_low[], double Ebin_high[],
			       double pseudo[], double pseudo_err[],
			       double brems[], double brems_err[],
			       double twoph[], double twoph_err[],
			       double radrec[], double radrec_err[]) {
  
  int tfields;
  char hduname[20];
  char *ttype[9], *tunit[9], *tform[9];
  float T_fl, ne_fl;
  float tot_emis, tot_brems, tot_rrc, tot_twoph, tot_pseudo;
  float pseudo_fl, pseudo_err_fl, brems_fl, brems_err_fl;
  float twoph_fl, twoph_err_fl, radrec_fl, radrec_err_fl;
  float Ebin_low_fl, Ebin_high_fl;
  int iBin, i, status=0;
  int extver;
  long nbin_l;
    
  T_fl = (float) T;
  ne_fl= (float) n_e;
  tot_emis  = 0.0;
  tot_brems = 0.0;
  tot_rrc   = 0.0;
  tot_twoph = 0.0;
  tot_pseudo= 0.0;
  for (iBin=0;iBin<nbin;iBin++) {
    tot_brems += brems[iBin]*(Ebin_high[iBin]-Ebin_low[iBin])*ERG_KEV;
    tot_rrc   += radrec[iBin]*(Ebin_high[iBin]-Ebin_low[iBin])*ERG_KEV;
    tot_twoph += twoph[iBin]*(Ebin_high[iBin]-Ebin_low[iBin])*ERG_KEV;
    tot_pseudo+= pseudo[iBin]*(Ebin_high[iBin]-Ebin_low[iBin])*ERG_KEV;
  }
  tot_emis = tot_brems + tot_rrc + tot_twoph + tot_pseudo;

  for(i=0;i<9;i++) {
    ttype[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
    tunit[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
    tform[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
  }
  
  tfields = 9;
  sprintf(ttype[0],"Ebin_high");
  sprintf(tform[0],"1E");
  sprintf(tunit[0],"keV");
  sprintf(ttype[1],"Pseudo ");
  sprintf(tform[1],"1E");
  sprintf(tunit[1],"phot cm^3 s^-1 bin^-1");
  sprintf(ttype[2],"Pseudo_Err");
  sprintf(tform[2],"1E");
  sprintf(tunit[2],"phot cm^3 s^-1 bin^-1");
  sprintf(ttype[3],"Brems");
  sprintf(tform[3],"1E");
  sprintf(tunit[3],"phot cm^3 s^-1 bin^-1 ");
  sprintf(ttype[4],"Brems_Err");
  sprintf(tform[4],"1E");
  sprintf(tunit[4],"phot cm^3 s^-1 bin^-1 ");
  sprintf(ttype[5],"Two_Photon");
  sprintf(tform[5],"1E");
  sprintf(tunit[5],"phot cm^3 s^-1 bin^-1 ");
  sprintf(ttype[6],"TwPh_Err");
  sprintf(tform[6],"1E");
  sprintf(tunit[6],"phot cm^3 s^-1 bin^-1 ");
  sprintf(ttype[7],"Rad_Rec");
  sprintf(tform[7],"1E");
  sprintf(tunit[7],"phot cm^3 s^-1 bin^-1 ");
  sprintf(ttype[8],"Rad_Rec_Err");
  sprintf(tform[8],"1E");
  sprintf(tunit[8],"phot cm^3 s^-1 bin^-1 ");
  
  if (n_e > 0) {
    sprintf(hduname,"C%.1f_%.1f",log10(T),log10(n_e));
  } else {
    sprintf(hduname,"C%.1f",log10(T));
  }    
  
  nbin_l = (long) nbin;
  if (fits_create_tbl(fptr, BINARY_TBL, nbin_l, tfields, ttype, tform, tunit,
		      "EMISSIVITY", &status)) {
    fits_printerror("Error creating binary HDU table",status);
  }

  fits_get_hdu_num(fptr, &extver);
  extver = extver-2;  /* Don't count PRIMARY or PARAMETER HDU; start at 1 */

  if (fits_write_key(fptr, TINT, "EXTVER", &extver,
		     "Spectral emission data", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }
  
  if (fits_write_key(fptr, TSTRING, "HDUNAME", hduname,
		     "Spectral emission data", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLASS", "SPECTRUM",
		     "Spectral data", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLAS1", "EMISSIVITY",
		     "Line and continuum emissivities", &status)) {
    fits_printerror("writing HDUCLAS1 keyword", status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLAS2", "CONTINUUM",
		     "Continuum tables", &status)) {
    fits_printerror("writing HDUCLAS2 keyword", status);
  }
  
  if (fits_write_key(fptr, TSTRING, "HDUVERS1", "0.9.0",
		     "version of format", &status)) {
    fits_printerror("writing HDUVERS1 keyword", status);
  }

  /* Write the parameters describing the table */
  
  if (fits_write_key(fptr, TFLOAT, "TEMPERATURE", &T_fl,"Electron temperature",
		     &status)) {
    fits_printerror("writing TEMPERATURE keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "DENSITY", &ne_fl,"Electron density",
		     &status)) {
    fits_printerror("writing DENSITY keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_EMIS", &tot_emis,
		     "Total Emission (erg cm^3 s^-1)", &status)) {
    fits_printerror("writing TOT_EMIS keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_BREM", &tot_brems,
		     "Total Bremsstrahlung Emission (erg cm^3 s^-1)",&status)){
    fits_printerror("writing TOT_BREM keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_RRC", &tot_rrc,
		     "Total RRC Emission (erg cm^3 s^-1)", &status)) {
    fits_printerror("writing TOT_RRC keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_TWOP", &tot_twoph,
		     "Total Two Photon Emission (erg cm^3 s^-1)", &status)) {
    fits_printerror("writing TOT_TWOP keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_PSEUDO", &tot_pseudo,
		     "Total PseudoContin. Emission (erg cm^3 s^-1)",&status)) {
    fits_printerror("writing TOT_PSEUDO keyword",status);
  }

  if (fits_write_key(fptr, TINT, "N_BIN", &nbin, 
			 "Number of energy bins", &status)) {
    fits_printerror("writing N_BIN keyword",status);
  }

  Ebin_low_fl = (float) Ebin_low[0];
  if (fits_write_key(fptr, TFLOAT, "MIN_E_BIN", &Ebin_low_fl, 
			 "Number of energy bins", &status)) {
    fits_printerror("writing N_BIN keyword",status);
  }
  
  for(iBin=0;iBin<nbin;iBin++) {
    
    Ebin_high_fl = (float) Ebin_high[iBin];
    pseudo_fl = (float) pseudo[iBin];
    if (pseudo_err != NULL) {
      pseudo_err_fl = (float) pseudo_err[iBin];
    } else {
      pseudo_err_fl = 0.0;
    }
    brems_fl = (float) brems[iBin];
    if (brems_err != NULL) {
      brems_err_fl = (float) brems_err[iBin];
    } else {
      brems_err_fl = 0.0;
    }
    twoph_fl = (float) twoph[iBin];
    if (twoph_err != NULL) {
      twoph_err_fl = (float) twoph_err[iBin];
    } else { 
      twoph_err_fl = 0.0;
    }
    radrec_fl = (float) radrec[iBin];
    if (radrec_err != NULL) {
      radrec_err_fl = (float) radrec_err[iBin];
    } else {
      radrec_err_fl = 0.0;
    }
  
    if (fits_write_col(fptr, TFLOAT, 1, iBin+1, 1, 1,&Ebin_high_fl,&status)){
      fits_printerror("writing energy bin",status);
    }

    if (fits_write_col(fptr, TFLOAT, 2, iBin+1, 1, 1, &pseudo_fl, &status)) {
      fits_printerror("writing pseudo-continuum emission line power",status);
    }

    if (fits_write_col(fptr, TFLOAT, 3, iBin+1, 1, 1,&pseudo_err_fl,&status)){
      fits_printerror("writing pseudo-continuum line power error",status);
    }

    if (fits_write_col(fptr, TFLOAT, 4, iBin+1, 1, 1,&brems_fl, &status)){
      fits_printerror("writing bremsstrahlung power",status);
    }

    if (fits_write_col(fptr, TFLOAT, 5, iBin+1, 1, 1,&brems_err_fl, &status)){
      fits_printerror("writing bremsstrahlung power error",status);
    }

    if (fits_write_col(fptr, TFLOAT, 6, iBin+1, 1, 1,&twoph_fl, &status)){
      fits_printerror("writing two photon power",status);
    }

    if (fits_write_col(fptr, TFLOAT, 7, iBin+1, 1, 1,&twoph_err_fl, &status)){
      fits_printerror("writing two photon power error",status);
    }

    if (fits_write_col(fptr, TFLOAT, 8, iBin+1, 1, 1,&radrec_fl, &status)){
      fits_printerror("writing radiative recombination power",status);
    }

    if (fits_write_col(fptr, TFLOAT, 9, iBin+1,1,1,&radrec_err_fl, &status)){
      fits_printerror("writing radiative recombination power error",status);
    }
  }

  for(i=0;i<9;i++) {
    free(ttype[i]);
    free(tunit[i]);
    free(tform[i]);
  }
}
