#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <fitsio.h>

#define MAXSTRLEN 128
#define KBOLTZ 8.617e-8
#define HC_IN_ERG_A 1.9848e-8

#define fits_write_colnull ffpcn

void fits_printerror(char *process, int status);

/****************************************************************************/
/* This routine takes an open fits file pointer (opened using               */
/* fits_init_specout) and writes line emission data.                        */
/* Inputs:                                                                  */
/* fptr       : fits file pointer for line file                             */
/* T          : Temperature of plasma (K)                                   */
/* n_e        : Electron density in plasma (cm^-3)                          */
/* nlines     : Total number of lines                                       */
/* lambda     : Array of line wavelengths, in Angstroms                     */
/* lambda_err : Associated error in these wavelengths, in A;-1 if Not Avail */
/* epsilon    : Array of line powers, in photons cm^3 / s                   */
/* epsilon_err: Associated error in line power, in photons cm^3/s,-1 if NA  */
/* el_Z       : Array of element Z values for lines in array                */
/* rmJ        : Ionization state, using roman numeral system,ie 1=neutral   */
/* UpperLev   : Index of upper level of transition, if available; -1 if not */
/* LowerLev   : Index of lower level of transition, if available; -1 if not */
/* If either lambda_err or epsilon_err is NULL, a value of -1 is assumed.   */
/* If either UpperLev or LowerLev is NULL, a value of -1 is assumed.        */
/****************************************************************************/

void fits_spec_write_lines(fitsfile *fptr, double T, double n_e, int nlines, 
			   double lambda[],  double lambda_err[], 
			   double epsilon[], double epsilon_err[], 
			   int el_Z[], int rmJ[],
			   int UpperLev[], int LowerLev[]) {
  
  int tfields;
  char hduname[20];
  char *ttype[8], *tunit[8], *tform[8];
  float T_fl, ne_fl, kT_fl;
  float tot_emission=0;
  float Lam_fl, Lerr_fl, Eps_fl, EpsErr_fl;
  float nulval=-1.0;
  int Z_uniq[100];
  int iLine, i, status=0;
  int ll, ul;
  int extver, chdu, NZ;
  long nlines_l, firstrow;

  for (iLine=0;iLine<nlines;iLine++) {
    tot_emission += (float) epsilon[iLine]*(HC_IN_ERG_A/lambda[iLine]);
  }

  kT_fl = (float) KBOLTZ*T;
  T_fl = (float) T;
  ne_fl= (float) n_e;

  for (i=0;i<100;i++) Z_uniq[i] = 0;
  
  for(i=0;i<8;i++) {
    ttype[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
    tunit[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
    tform[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
  }

  tfields = 8;
  sprintf(ttype[0],"Lambda");
  sprintf(tform[0],"1E");
  sprintf(tunit[0],"A");
  sprintf(ttype[1],"Lambda_Err");
  sprintf(tform[1],"1E");
  sprintf(tunit[1],"A");
  sprintf(ttype[2],"Epsilon");
  sprintf(tform[2],"1E");
  sprintf(tunit[2],"photons cm^3 s^-1");
  sprintf(ttype[3],"Epsilon_Err");
  sprintf(tform[3],"1E");
  sprintf(tunit[3],"photons cm^3 s^-1");
  sprintf(ttype[4],"Element");
  sprintf(tform[4],"1J");
  sprintf(tunit[4]," ");
  sprintf(ttype[5],"Ion");
  sprintf(tform[5],"1J");
  sprintf(tunit[5]," ");
  sprintf(ttype[6],"UpperLev");
  sprintf(tform[6],"1J");
  sprintf(tunit[6]," ");
  sprintf(ttype[7],"LowerLev");
  sprintf(tform[7],"1J");
  sprintf(tunit[7]," ");

  if (n_e > 0) {
    sprintf(hduname,"L%.1f_%.1f",log10(T),log10(n_e));
  } else {
    sprintf(hduname,"L%.1f",log10(T));
  }    
  nlines_l = (long) nlines;
  
  if (fits_create_tbl(fptr, BINARY_TBL, nlines_l, tfields, ttype,
		      tform, tunit, "EMISSIVITY", &status)) {
    fits_printerror("creating table extension",status);
  }
  
  fits_get_hdu_num(fptr, &extver);
  extver = extver-2;  /* Don't count PRIMARY or PARAMETER HDU; start at 1 */

  if (fits_write_key(fptr, TINT, "EXTVER", &extver,
		     "Index for this EMISSIVITY extension", &status)) {
    fits_printerror("writing EXTVER keyword",status);
  }
  
  if (fits_write_key(fptr, TSTRING, "HDUNAME", hduname,
		     "Spectral emission data", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLASS", "Proposed OGIP",
		     "Proposed OGIP standard", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLAS1", "LINE MODEL",
		     "Line emission spectral model", &status)) {
    fits_printerror("writing HDUCLAS1 keyword", status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLAS2", "LINE",
		     "Emission line data", &status)) {
    fits_printerror("writing HDUCLAS2 keyword", status);
  }
  
  if (fits_write_key(fptr, TSTRING, "HDUVERS1", "1.0.0",
		     "version of format", &status)) {
    fits_printerror("writing HDUVERS1 keyword", status);
  }

  /* Write the parameters describing the table */
  
  if (fits_write_key(fptr, TFLOAT, "TEMPERATURE", &T_fl,"Electron temperature",
		     &status)) {
    fits_printerror("writing TEMPERATURE keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "DENSITY", &ne_fl,"Electron density",
		     &status)) {
    fits_printerror("writing DENSITY keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_LINE", &tot_emission,
		     "Total Line Emission (erg cm^3 s^-1)", &status)) {
    fits_printerror("writing TOT_LINE keyword",status);
  }

  if (fits_write_key(fptr, TINT, "N_LINES", &nlines, 
			 "Number of emission lines", &status)) {
    fits_printerror("writing N_LINES keyword",status);
  }

  for(iLine=0;iLine<nlines;iLine++) {
    
    Lam_fl   = lambda[iLine];
    Eps_fl   = epsilon[iLine];
    if (lambda_err != NULL) { 
      Lerr_fl=lambda_err[iLine];
      if (Lerr_fl <= 0.0) Lerr_fl = nulval;
    } else { 
      Lerr_fl=nulval;
    }
    if (epsilon_err != NULL) { 
      EpsErr_fl=epsilon_err[iLine];
    } else { 
      EpsErr_fl=nulval;
    }
    if (UpperLev != NULL) { 
      ul = UpperLev[iLine];
    } else { 
      ul = -1;
    }
    if (LowerLev != NULL) { 
      ll= LowerLev[iLine];
    } else { 
      ll=-1;
    }
    Z_uniq[el_Z[iLine]] = 1;
    
    /* Plam_fl = 1.e-23*Plambda[iLine]/(1.9848e-8/lambda[iLine]); */
    /* Plerr_fl = Plambda_err[iLine]/(1.9848e-8/lambda[iLine]);   */
    
  
    if (fits_write_col(fptr, TFLOAT, 1, iLine+1, 1, 1, &Lam_fl, &status)) {
      fits_printerror("writing lambda",status);
    }

    if (fits_write_colnull(fptr, TFLOAT, 2, iLine+1, 1, 1, &Lerr_fl, &nulval, 
			  &status)) {
      fits_printerror("writing lambda error",status);
    }

    if (fits_write_col(fptr, TFLOAT, 3, iLine+1, 1, 1, &Eps_fl, &status)) {
      fits_printerror("writing emission line emissivity", status);
    }
    
    if (fits_write_colnull(fptr, TFLOAT, 4, iLine+1, 1, 1, &EpsErr_fl, &nulval,
			  &status)) {
      fits_printerror("writing emission line emissivity error",status);
    }

    if (fits_write_col(fptr, TINT, 5, iLine+1, 1, 1, &(el_Z[iLine]),&status)) {
      fits_printerror("writing element number",status);
    }

    if (fits_write_col(fptr, TINT, 6, iLine+1, 1, 1, &(rmJ[iLine]),&status)){
      fits_printerror("writing ionization value (1 = neutral)",status);
    }

    if (fits_write_col(fptr, TINT, 7,iLine+1, 1, 1, &ul, &status)) {
      fits_printerror("writing upper energy level",status);
    }

    if (fits_write_col(fptr, TINT, 8, iLine+1, 1, 1, &ll, &status)) {
      fits_printerror("writing lower energy level",status);
    }
  }
  
  for(i=0;i<8;i++) {
    free(ttype[i]);
    free(tunit[i]);
    free(tform[i]);
  }

  /* Now, jump back to the first BINARY HDU and write the necessary info. */
  /* Start by saving the current HDU so we can return to it */

  fits_get_hdu_num(fptr, &chdu);
  
  /* Jump to the PARAMETERS hdu */
  if (fits_movabs_hdu(fptr, 2, NULL, &status)) {
    fits_printerror("moving to PARAMETERS hdu.",status);
  }
  
  /* Write out the relevant row data; row number is, conveniently, = extver. */
  firstrow = (long) extver;
  
  if (fits_write_col(fptr, TFLOAT, 1, firstrow, 1, 1, &kT_fl, &status)) {
    fits_printerror("writing temperature, in keV",status);
  }

  if (fits_write_col(fptr, TFLOAT, 2, firstrow, 1, 1, &ne_fl, &status)) {
    fits_printerror("writing electron density, in cm^-3",status);
  }
  NZ = 0;
  for (i=0;i<100;i++) NZ = NZ + Z_uniq[i];

  if (fits_write_col(fptr, TINT, 3, firstrow, 1, 1, &NZ, &status)) {
    fits_printerror("writing number of elements",status);
  }

  if (fits_write_col(fptr, TINT, 4, firstrow, 1, 1, &nlines, &status)) {
    fits_printerror("writing number of lines",status);
  }

  /* And now jump back to the most recent HDU */
  if (fits_movabs_hdu(fptr, chdu, NULL, &status)) {
    fits_printerror("returning to current HDU.",status);
  }

}

void f77_fits_spec_write_lines(fitsfile *fptr, double *T, double *n_e,
			       int *nlines, double lambda[], 
			       double lambda_err[], double epsilon[], 
			       double epsilon_err[], int el_Z[], int rmJ[],
			       int UpperLev[], int LowerLev[]) {
    
  fits_spec_write_lines(fptr, *T, *n_e, *nlines, lambda, lambda_err, 
			epsilon, epsilon_err, el_Z, rmJ, UpperLev, LowerLev);

}
