#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "calc_maxwell_rates.h"
#include "messages.h"

/*! \file calc_maxwell_rates.c 
 *  \brief Calculates Maxwellian-averaged collision strengths 
 */

void prep_spline(double x[], double y[], int n, double yp[]);
double calc_spline(double xa[], double ya[], double y2a[], int n, double x);
double interpol_huntd(int n, double *x, double *y, double z);
double calc_kato(int colltype, double om[], int Z, double Te);
double calc_sampson_s(double om[], int Z, double Te);
double calc_sampson_p(double om[], int Z, double Te);
double calc_sampson_h(double om[], int Z, double Te);
double exint1(double x, int jump);

  /*! \brief Calculates Maxwellian-averaged collision strengths.
    Calculates Maxwellian-averaged collision strengths for any 
    transition, given input values taken from the APED (collision type, 
    temperature minimum/maximum, and data vectors Tarr and om.  Also 
    requires the energy separating the two levels, the electron (or proton)
    temperature, the proton number, and the upper and lower level 
    degeneracies.  Returns both the excitation and the deexcitation rate.
    \param coll_type The collision type, as defined in calc_maxwell_rates.h
    \param min_T The minimum temperature (in Kelvin) for which the 
    transition is defined. 
    \param max_T The maximum temperature (in Kelvin) for which the 
    transition is defined.
    \param Tarr
    \param om
    \param dE The delta Energy of the transition, in keV
    \param T  Temperature, in Kelvin, of either the electrons or protons 
    involved in the transition.
    \param Z  The # of protons in the target ion.
    \param degl The degeneracy of the lower level in the transition
    \param degu The degeneracy of the upper level in the transition
    \param exc_rate The calculated excitation rate coefficient, in cm^3/s.
    \param dex_rate The calculated deexcitation rate coefficient, in cm^3/s.
  */

void calc_maxwell_rates(int coll_type, double min_T, double max_T, 
			double Tarr[], double om[], double dE, double T,
			int Z, double degl, double degu, 
			double *exc_rate, double *dex_rate) {
 
  int calc_type, N_interp, num_spline;
  double a,b,c,d,e;
  double chi, chi_inv, upsilon, rate_coeff,  st, logT, expint_2;
  double xs[5] = {0.00, 0.25, 0.50, 0.75, 1.00};
  double xs9[9] = {0.00, 0.125, 0.25, 0.375, 0.50, 0.675, 0.80, 0.925, 1.00};
  double yp[5], yp9[9], *spline_dat;


  calc_type = -1;
  chi  = dE / (KBOLTZ*T);
  chi_inv = (KBOLTZ*T) / dE;  

  logT = log10(T);
  st = 0;
  upsilon = 0.0;
  rate_coeff = 0.0;

  *exc_rate = 0;
  *dex_rate = 0;

  /* Check to make sure temperature is inside valid range for this */
  /* transition; otherwise return defaults set above.  */
  if ((T < min_T)||(T > max_T)) {
    /* message("calc_maxwell_rates","Bad transition; %e < %e or %e > %e",
       T, min_T, T,max_T); */
    return;
  }

  if (coll_type == BURGESSTULLY) {
    expint_2 = exint1(chi,2);
    a = om[0];
    b = om[1];
    c = om[2];
    d = om[3];
    e = om[4];
    upsilon = a + b*chi*expint_2 + c*chi*(1-chi*expint_2)
      + d*(chi/2)*(1-chi*(1-chi*expint_2)) + e*expint_2;
    calc_type = E_UPSILON;
  }

  if ((coll_type == CHIANTI_1)||
      (coll_type == CHIANTI_2)||
      (coll_type == CHIANTI_3)||
      (coll_type == CHIANTI_4)||
      (coll_type == CHIANTI_5)||
      (coll_type == CHIANTI_6)) {
    c = om[0];

    if (coll_type == CHIANTI_1) st = 1 - (log(c)/log(chi_inv+c));
    if (coll_type == CHIANTI_2) st = chi_inv/(chi_inv+c);
    if (coll_type == CHIANTI_3) st = chi_inv/(chi_inv+c);
    if (coll_type == CHIANTI_4) st = 1 - (log(c)/log(chi_inv+c));
    if (coll_type == CHIANTI_5) st = chi_inv/(chi_inv+c);
    if (coll_type == CHIANTI_6) st = chi_inv/(chi_inv+c);

    spline_dat = &(om[1]);
    prep_spline(xs, spline_dat, 5, yp);
    upsilon = calc_spline(xs, spline_dat, yp, 5, st);

    if (coll_type == CHIANTI_1) upsilon *= log(chi_inv + M_E);
    if (coll_type == CHIANTI_2) ;  /* Do nothing */
    if (coll_type == CHIANTI_3) upsilon /= (chi_inv+1);
    if (coll_type == CHIANTI_4) upsilon *= log(chi_inv + c);
    if (coll_type == CHIANTI_5) upsilon /= chi_inv;
    if (coll_type == CHIANTI_6) upsilon = pow(10.,upsilon);
    if (coll_type == CHIANTI4_6) {
      calc_type = P_UPSILON; } else { calc_type = E_UPSILON; }
  }

  if ((coll_type == CHIANTI4_1)||
      (coll_type == CHIANTI4_2)||
      (coll_type == CHIANTI4_3)||
      (coll_type == CHIANTI4_4)||
      (coll_type == CHIANTI4_5)||
      (coll_type == CHIANTI4_6)) {
    num_spline = (int) om[0];
    c = om[1];

    if (coll_type == CHIANTI4_1) st = 1 - (log(c)/log(chi_inv+c));
    if (coll_type == CHIANTI4_2) st = chi_inv/(chi_inv+c);
    if (coll_type == CHIANTI4_3) st = chi_inv/(chi_inv+c);
    if (coll_type == CHIANTI4_4) st = 1 - (log(c)/log(chi_inv+c));
    if (coll_type == CHIANTI4_5) st = chi_inv/(chi_inv+c);
    if (coll_type == CHIANTI4_6) st = chi_inv/(chi_inv+c);

    spline_dat = &(om[2]);
    if (num_spline == 5) {
      prep_spline(xs, spline_dat, num_spline, yp);
      upsilon = calc_spline(xs, spline_dat, yp, num_spline, st);
    } else {
      if (num_spline == 9) {
	prep_spline(xs9, spline_dat, num_spline, yp9);
	upsilon = calc_spline(xs9, spline_dat, yp9, num_spline, st);
      } else {
	errmess("calc_maxwell_rates",
		"Chianti data with %d values, not 5 or 9 as expected",
		num_spline);
      }
    } 

    if (coll_type == CHIANTI4_1) upsilon *= log(chi_inv + M_E);
    if (coll_type == CHIANTI4_2) ;  /* Do nothing */
    if (coll_type == CHIANTI4_3) upsilon /= (chi_inv+1);
    if (coll_type == CHIANTI4_4) upsilon *= log(chi_inv + c);
    if (coll_type == CHIANTI4_5) upsilon /= chi_inv;
    if (coll_type == CHIANTI4_6) upsilon = pow(10.,upsilon);
    if (coll_type == CHIANTI4_6) {
      calc_type = P_UPSILON; } else { calc_type = E_UPSILON; }
  }

  if (coll_type == SGC_1) {  /* S-type transitions, He-like */
    upsilon = calc_sampson_s(om, Z, T);
    calc_type = E_UPSILON;
  }

  if (coll_type == SGC_2) {  /* P-type transitions, He-like */
    upsilon = calc_sampson_p(om, Z, T);
    calc_type = E_UPSILON;
  }

  if (coll_type == SGC_3) {  /* S-type transitions, H-like */
    upsilon = calc_sampson_h(om, Z, T);
    calc_type = E_UPSILON;
  }

  if (coll_type == KATO_NAKAZAKI_1) {
    upsilon = calc_kato(1, om, Z, T);
    calc_type = E_UPSILON;
  }

  if (coll_type == KATO_NAKAZAKI_2) {
    upsilon = calc_kato(2, om, Z, T);
    calc_type = E_UPSILON;
  }

  if (coll_type == PROTON_BT) {
    a = om[0];
    b = om[1];
    c = om[2];
    /* 0.8 for the np/ne ratio, */
    if ((logT > om[3])&&( logT < om[4])) {
      rate_coeff=0.8*pow(10., (a + b*(logT) + c*logT*logT) );
    }
    calc_type = P_RATE_COEFF;
  }
  
  if ((coll_type >= INTERP_E_UPSILON) && 
      (coll_type <= INTERP_E_UPSILON + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_UPSILON;
    upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_UPSILON;
  }

  if ((coll_type >= INTERP_E_UPS_OPEN) && 
      (coll_type <= INTERP_E_UPS_OPEN + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_UPS_OPEN;
    if ((T > min_T)&&(T < max_T)) 
      upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_UPSILON;
  }

  if ((coll_type >= INTERP_E_UPS_INC_MIN) && 
      (coll_type <= INTERP_E_UPS_INC_MIN + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_UPS_INC_MIN;
    if (T < max_T) 
      upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_UPSILON;
  }

  if ((coll_type >= INTERP_E_UPS_INC_MAX) && 
      (coll_type <= INTERP_E_UPS_INC_MAX + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_UPS_INC_MAX;
    if (T > min_T) 
      upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_UPSILON;
  }

  if ((coll_type >= INTERP_P_UPSILON) && 
      (coll_type <= INTERP_P_UPSILON + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_UPSILON;
    upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_UPSILON;
  }

  if ((coll_type >= INTERP_P_UPS_OPEN) && 
      (coll_type <= INTERP_P_UPS_OPEN + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_UPS_OPEN;
    if ((T > min_T)&&(T < max_T)) 
      upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_UPSILON;
  }

  if ((coll_type >= INTERP_P_UPS_INC_MIN) && 
      (coll_type <= INTERP_P_UPS_INC_MIN + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_UPS_INC_MIN;
    if (T < max_T) 
      upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_UPSILON;
  }

  if ((coll_type >= INTERP_P_UPS_INC_MAX) && 
      (coll_type <= INTERP_P_UPS_INC_MAX + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_UPS_INC_MAX;
    if (T > min_T) 
      upsilon = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_UPSILON;
  }

  if ((coll_type >= INTERP_E_RATE_COEFF) && 
      (coll_type <= INTERP_E_RATE_COEFF + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_RATE_COEFF;
    rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_RATE_COEFF;
  }

  if ((coll_type >= INTERP_E_RATE_OPEN) && 
      (coll_type <= INTERP_E_RATE_OPEN + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_RATE_OPEN;
    if ((T > min_T)&&(T < max_T)) 
      rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_RATE_COEFF;
  }

  if ((coll_type >= INTERP_E_RATE_INC_MIN) && 
      (coll_type <= INTERP_E_RATE_INC_MIN + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_RATE_INC_MIN;
    if (T < max_T) 
      rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_RATE_COEFF;
  }

  if ((coll_type >= INTERP_E_RATE_INC_MAX) && 
      (coll_type <= INTERP_E_RATE_INC_MAX + MAX_UPS)) {
    N_interp = coll_type - INTERP_E_RATE_INC_MAX;
    if (T > min_T) 
      rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = E_RATE_COEFF;
  }

  if ((coll_type >= INTERP_P_RATE_COEFF) && 
      (coll_type <= INTERP_P_RATE_COEFF + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_RATE_COEFF;
    rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_RATE_COEFF;
  }

  if ((coll_type >= INTERP_P_RATE_OPEN) && 
      (coll_type <= INTERP_P_RATE_OPEN + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_RATE_OPEN;
    if ((T > min_T)&&(T < max_T))
      rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_RATE_COEFF;
  }

  if ((coll_type >= INTERP_P_RATE_INC_MIN) && 
      (coll_type <= INTERP_P_RATE_INC_MIN + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_RATE_INC_MIN;
    if (T < max_T) 
      rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_RATE_COEFF;
  }

  if ((coll_type >= INTERP_P_RATE_INC_MAX) && 
      (coll_type <= INTERP_P_RATE_INC_MAX + MAX_UPS)) {
    N_interp = coll_type - INTERP_P_RATE_INC_MAX;
    if (T > min_T) 
      rate_coeff = interpol_huntd(N_interp, Tarr, om, T);
    calc_type = P_RATE_COEFF;
  }
  
  if (calc_type == -1) {
    errmess("calc_trans_rates","Undefined collision type.");
  }

  if (calc_type == E_UPSILON) {
    
    if (upsilon <= 0.)  {
      upsilon = 0.; /* Negative upsilon is unphysical. */
    }
    
    if (chi < MAX_CHI) {
      *exc_rate = UPSILON_COLL_COEFF * upsilon * exp(-chi) / (sqrt(T)*degl);
      *dex_rate = UPSILON_COLL_COEFF * upsilon / (sqrt(T)*degu);
    }
  }
  
  if (calc_type == P_UPSILON) {
    
    if (upsilon <= 0.)  {
      upsilon = 0.; /* Negative upsilon is unphysical. */
    }
    
    *exc_rate = 0.;
    *dex_rate = 0.;
    message("calc_rate","Can't calculate collision strength for protons.");
  }
  
  if ((calc_type == P_RATE_COEFF)||(calc_type == E_RATE_COEFF)) {
    if (rate_coeff < 0) {
      rate_coeff= 0.; /* Negative rate coefficient is unphysical. */
    }
    *exc_rate = rate_coeff;
    *dex_rate = rate_coeff*exp(chi)*(degl/degu);
  }
  
}
  
