#include <math.h>

#include "apec_macros.h"
#include "apec.h"
#include "messages.h"

double kurucz(double uin, double gam);

void brems_gaunt_kellogg(double *E, double T, int Z, int N, double *g_ff) {

  /* Initialized data */

  double gam2[6] = { .7783,1.2217,2.6234, 4.3766,20.,70. };
  double gam3[6] = { 1.,1.7783,3.,5.6234, 10.,30. };
  double a[126]={ 1.001, 1.004, 1.017, 1.036, 1.056, 1.121, 1.001, 1.005, 
		  1.017, 1.046, 1.073, 1.115, .9991, 1.005, 1.03, 1.055, 
		  1.102, 1.176, .997, 1.005, 1.035,  1.069, 1.134, 1.186, 
		  .9962, 1.004, 1.042, 1.1, 1.193, 1.306,.9874, .9962, 
		  1.047, 1.156, 1.327, 1.485, .9681, .9755, 1.02, 1.208, 
		  1.525, 1.965, .3029, .1616, .04757, .013, .0049, -.0032,
		  .4905, .2155, .08357, .02041, .00739, 2.9e-4, .654, 
		  .2833, .08057, .03257, .00759, -.00151, 1.029, .391, 
		  .1266, .05149, .01274, .00324, .9569, .4891, .1764, 
		  .05914, .01407, -2.4e-4, 1.236, .7579, .326, .1077, 
		  .028, .00548, 1.327, 1.017,.6017, .205, .0605, .00187, 
		  -1.323, -.254, -.01571, -.001, -1.84e-4, 8e-5, -4.762, 
		  -.3386, -.03571, -.001786, -3e-4, 1e-5, -6.349, -.4206, 
		  -.02571, -.003429, -2.34e-4, 5e-5, -13.231, -.59, 
		  -.04571, -.005714, -4.45e-4, -4e-5, -7.672, -.6852, 
		  -.0643, -.005857,  -4.2e-4, 4e-5, -7.143, -.9947, 
		  -.12, -.01007, -8.51e-4, -4e-5, -3.175, -1.116, -.227, 
		  -.01821, -.001729, 2.3e-4 };
  
  
  /* Local variables */
  double u12im, born, g, kT;
  int m, n, iBin;
  double p, t, u;
  double g1, g2;
  int m1;
  double u1, u2, t2, t4, t8, ai, ak, u12, u122;
  double gam, gam1;
  /* From XSPEC v 10.0 */
  /* Calculation of the Gaunt factor based on the program given in */
  /* Kellogg, Baldwin, & Koch (ApJ 199, 299). Fixes supplied by Larry */
  /* Molnar and Jack Hughes for inaccuracies in the polynomial fits */
  /* to the Karzas & Latter (1961) numerical values. The routine by */
  /* Kurucz for the high T case was supplied by Larry Molnar. */
  /* arguments (actually, this is the LOW T case) */
  /* 		e	r	i:energy (keV) */
  /* 		q	r	i: temperature (keV) */
  /* 		z	r	i: ion charge used for correction */
  /* 	returned value */
  /* 		gaunt	r	r: the gaunt factor */
  
  /* Convert the energy and temperature to the units used by Karzas & Latter
   */
  kT = KBOLTZ*T;
  gam = Z * Z * RYDBERG / kT;
  gam1 = min(gam*1.e3,100.);

  /* If we are in the low T regime then use Kurucz's algorithm */
  if (gam > .1) {
    for (iBin=0;iBin<N;iBin++) g_ff[iBin] = kurucz(E[iBin]/kT, gam);
    return;
  }

  /* if temperature or energy is zero or if energy/temperature is */
  /* too high then give up. */
  if (kT == 0.) {
    message("brems_kellog","Zero temperature!");
    for (iBin=0;iBin<N;iBin++) g_ff[iBin] = 0.0;
    return;
  }

  for (iBin=0;iBin<N;iBin++) {
    u = E[iBin] / kT;

    if ((u > 50) || (u == 0.)) {
      g_ff[iBin] = 0.;
    } else {
    
      /* Calculate the Born approximation */
      g = 1.;
      u2 = u * u;
      u1 = u * .5;
      t = u1 / 3.75;
      u12 = u1 * .5;
      if (u12 <= 1.) {
	t2 = t * t;
	t4 = t2 * t2;
	t8 = t4 * t4;
	ai = t2 * 3.5156229 + 1. + t4 * 3.089942 + t2 * 1.2067492 * t4 +
	  t8 * .2659732 + t8 * .0360768 * t2 + t8 * .0045813 * t4;
	u122 = u12 * u12;
	ak = -log(u12) * ai - .57721566 + u122 *
	  (u122 * (u122 * (u122 * (u122 * (u122 * 7.4e-6 + 1.075e-4) + 
				   .00262698) + .0348859) + .23069756) + 
	   .4227842);
      } else {
	u12im = -1. / u12;
	ak = u12im*(u12im*(u12im*(u12im*(u12im*(u12im*5.3208e-4 + .0025154)
					 + .00587872) + .01062446)
			   + .02189568) + .07832358) + 1.25331414;
	ak /= exp(u1) * sqrt(u1);
      }
      born = exp(u1) * .5513 * ak;
      /* If Born approximation is valid then go with it */
      if (gam1 < 1.) {
	g_ff[iBin] = born;
      } else {
      
	/* All that is left to do is the polynomial approximation */
	if (u < .003) {
	  u = .003;
	  u2 = u * u;
	}
	if (u <= .03) {	        n = 1;
	} else if (u <= .3) {	n = 2;
	} else if (u <= 1.) {	n = 3;
	} else if (u <= 5.) {	n = 4;
	} else if (u <= 15.) {	n = 5;
	} else {             	n = 6;
	}
	if (gam1 <= 1.773) {	   m = 1;
	} else if (gam1 <= 3.) {	   m = 2;
	} else if (gam1 <= 5.6234) { m = 3;
	} else if (gam1 <= 10.) {    m = 4;
	} else if (gam1 <= 30.) {    m = 5;
	} else {                     m = 6;
	}       
	m1 = m + 1;
	g1 = (a[n + (m + 7) * 6 - 49] + 
	      a[n + (m + 14) * 6 - 49] * u + 
	      a[n + (m + 21) * 6 - 49] * u2) * born;
	g2 = (a[n + (m1 + 7) * 6 - 49] + 
	      a[n + (m1 + 14) * 6 - 49] * u + 
	      a[n + (m1 + 21) * 6 - 49] * u2) * born;
	p = (gam1 - gam3[m - 1]) / gam2[m - 1];
	
	g_ff[iBin] = (1. - p) * g1 + p * g2;
      }
    }
  }
} 

/*  routine for high T case (from Larry Molnar) */

double kurucz(double uin, double gam) {

  static double ya[84] = { 5.4,5.25, 5.,4.69,4.48,4.16,3.85,4.77,4.63,4.4,
			   4.13,3.87,3.52, 3.27,4.15,4.02,3.8,3.57,3.27,
			   2.98,2.7,3.54,3.41,3.22,2.97,2.7,2.45,2.2,2.94,
			   2.81,2.65,2.44,2.21,2.01,1.81,2.41,2.32,2.19,2.02,
			   1.84,1.67,1.5,1.95,1.9,1.8,1.68,1.52,1.41,1.3,
			   1.55,1.56,1.51,1.42,1.33,1.25,1.17,1.17,1.3,1.32,
			   1.3,1.2,1.15,1.11,.86,1.,1.15,1.18,1.15,1.11,1.08,
			   .59,.76,.97,1.09,1.13,1.1,1.08,.38,.53,.76,.96,
			   1.08,1.09,1.09 };

  /* Local variables */
  int j, k;
  double t, u, rj, rk;
  double gaunt;

  rj = log10(gam) * 2. + 3.;
  j = rj;
  rj =  j;
  rk = log10(uin) * 2. + 9.;
  k = rk;
  if (k < 1)  k = 1;
  rk = k;
  if (j >= 7 || j < 1 || k >= 12) {
    gaunt = 0.;
  } else {
    t = (log10(gam) - (rj - 3.) / 2.) / .5;
    u = (log10(uin) - (rk - 9.) / 2.) / .5;
    gaunt = (1. - t) * (1. - u) * ya[j + k * 7 - 8] + 
      t * (1.-u)* ya[j + 1 + k*7 - 8] + t*u*ya[j + 1 + (k + 1) * 7 - 8] +
      (1. - t) * u * ya[j + (k + 1) * 7 - 8];
  }
  return gaunt;
} /* kurucz_ */

