#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <fitsio.h>

#include "messages.h"
#include "apec.h"
#include "apec_macros.h"
#include "datatypes.h"

/*! \file bremsstrahlung.c */

#define BREMS_COEFF 1.6542996e-20

void brems_gaunt_hummer(double *E, double T, int Z, int N, double *g_ff);
void brems_gaunt_kellogg(double *E, double kT, int Z, int N, double *g_ff);
void brems_gaunt_relativistic(double *E, double T, int Z, int N, 
			      double *g_ff);
void init_brems_hummer(char *filemap_file);
void init_brems_relativistic(char *filemap_file);

void compress_energy_spectrum(int Nbin, double *bin_edge, double *emission, 
			      double error, char *type,
			      int *Nval, double **Eval, double **Fluxval);

/*! \brief Calculate bremsstrahlung emission. 
 *  \param Z proton number of element under consideration
 *  \param rmJ Ion number (neutral = 1) of ion
 *  \param abund Abundance (relative to Hydrogen of element 
 *  \param Ion_pop[] Ionization population of ion under consideration
 *  \param filemap_file Filemap file, used to initialize data as needed
 *  \param Brems_Type Type of bremsstrahlung; Hummer, Kellogg, Relativistic
 *  \param T electron temperature (in Kelvin)
 *  \param spectrum Spectrum structure
 * Following the discussion in Sutherland, 1998, MNRAS, 300, 321 
 * Equation (15): 
    P_ff(\nu, T)d\nu = 
         n_e n_H f_k (exp(-u)/\sqrt{T(K)})\sum A_el \chi_el,Z Z^2
     \times \langle g_ff(\gamma^2, u) \rangle d\nu erg cm^{-3} s^{-1} 
 * Where f_k={{16}\over{3\sqrt{3}}} \sqrt{{\pi}\over{2km_e^3}}(e^2/c)^3 
 *   = {{16}\over{3\sqrt{3}}} \sqrt{{\pi m_e^3}\over{2k}} c^3 r_e^3 
 *   = 5.44436e-39 erg cm^3 / s 
 * More convenient than dv, in Hz, however, is dE, and E = hv.  
 * h = 4.135669e-18 keV s 
 * Converting to per keV units, we get :     
 *  f_k = 1.316439976e-21 erg cm^3 / s / keV / sr 
 * Converting to total emission (multiply by 4pi) 
 *  f_k = 1.6542996e-20 erg cm^3 / s / keV 
 */
 
void bremsstrahlung(int Z, int rmJ, double abund, double Ion_pop,
		    char *filemap_file, int Brems_Type, double T, 
		    struct BIN_SPEC *spectrum){

  
  double kT, *E, *EkT, *dE;  /* in keV */
  double *emission;
  double *gaunt_ff;  /* Free-free gaunt factor */
  double tot_emis=0.0;
  int iBin;
  struct COMPRESS_CONT *cs_ptr;

  if (spectrum==NULL) return;

  kT = KBOLTZ*T;

  if (Brems_Type == HUMMER) init_brems_hummer(filemap_file);
  if (Brems_Type == RELATIVISTIC) init_brems_relativistic(filemap_file);

  emission = (double *) malloc((spectrum->Nbin)*sizeof(double));
  E  = (double *) malloc((spectrum->Nbin)*sizeof(double));
  dE = (double *) malloc((spectrum->Nbin)*sizeof(double));
  EkT= (double *) malloc((spectrum->Nbin)*sizeof(double));
  gaunt_ff = (double *) malloc((spectrum->Nbin)*sizeof(double));

  for (iBin=0;iBin<spectrum->Nbin;iBin++) {
    E[iBin]  =0.5*((spectrum->bin_edge)[iBin]+(spectrum->bin_edge)[iBin+1]);
    dE[iBin] =(spectrum->bin_edge)[iBin+1]-(spectrum->bin_edge)[iBin];
    EkT[iBin]=E[iBin]/kT;
    gaunt_ff[iBin] = 0.0;
  }

  switch(Brems_Type) {
  case (HUMMER):
    brems_gaunt_hummer(E, T, rmJ-1, spectrum->Nbin, gaunt_ff);
    break;
  case (KELLOGG):
    brems_gaunt_kellogg(E, T, rmJ-1, spectrum->Nbin, gaunt_ff);
    break;
  case (RELATIVISTIC):
    brems_gaunt_relativistic(E, T, rmJ-1, spectrum->Nbin, gaunt_ff);
    break;
  case (BREMS_NONE):
    /* Do nothing */
    break;
  default:
    errmess("bremsstrahlung","Illegal bremsstrahlung type.");
    break;
  }
  
  for (iBin=0;iBin<spectrum->Nbin;iBin++) {
    
    emission[iBin] = BREMS_COEFF*abund*Ion_pop*
      (rmJ-1)*(rmJ-1)*exp(-EkT[iBin])*(dE[iBin]/sqrt(T))*gaunt_ff[iBin];

    (spectrum->brems_emiss[Z])[iBin] += (emission[iBin]/(E[iBin]*ERG_KEV));
  }

  cs_ptr = spectrum->cs;
  while ((cs_ptr != NULL) && ((cs_ptr->Z != Z) || (cs_ptr->rmJ != rmJ))) {
    cs_ptr = cs_ptr->next;
  }
  if (cs_ptr == NULL) 
    errmess("bremsstrahlung","Ion spectrum for %d, %d not found?",Z,rmJ);
  for (iBin=0;iBin<spectrum->Nbin;iBin++) 
    cs_ptr->bin_real[iBin] += emission[iBin];
  compress_energy_spectrum(spectrum->Nbin, spectrum->bin_edge, emission, 
			   ERROR_BREMS, "Bremsstrahlung",&(cs_ptr->Nbrems), 
			   &(cs_ptr->E_brems), &(cs_ptr->brems));

  for (iBin=0;iBin<spectrum->Nbin;iBin++) {
    (spectrum->bin_emission[Z])[iBin] += (spectrum->brems_emiss[Z])[iBin];
    tot_emis += emission[iBin];
  }

  spectrum->total_emission[Z] += tot_emis; /* ergs cm^3 / s */
  spectrum->tot_brems_emis[Z] += tot_emis; /* ergs cm^3 / s */

  free(emission);
  free(E);
  free(dE);
  free(EkT);
  free(gaunt_ff);
}
