#include <stdlib.h>
#include <stdio.h>

#include "messages.h"
#include "apec.h"

/*! \file calc_rad_rec_cont.c */

void compress_energy_spectrum(int Nbin, double *bin_edge, double *emission, 
			      double error, char *type,
			      int *Nval, double **Eval, double **Fluxval);
void calc_rrc(int Z, int rmJ, double abund, double ion_pop, 
	      double I_e, double g_ratio, int sig_type, double *sigma_coeff, 
	      double T, double min_emission, 
	      struct BIN_SPEC *spectrum, double *emission,double *rr_lev_rate);
char *roman_element(int Z, int J);

  /*! \brief Calculate total radiative recombination and continuum 
   *  \param debug Debugging output files [optional; set to NULL to ignore]
   *  \param Z proton number of element 
   *  \param rmJ ion stage (neutral=1) of recombined ion
   *  \param abund Abundance of element Z
   *  \param this_ion_pop Ion population of recombined ion (unitless)
   *  \param pstate Atomic data for parent (recombining)ion 
   *  \param state Atomic data for recombined ion
   *  \param Te electron temperature, in Kelvin 
   *  \param LevelRecombRate Output vector of total recombination rate to each level 
   *  \param spectrum Spectral output structure [optional; set to NULL to ignore]
   *  The code sets up the call to calc_rrc which does the actual calculation.
   *  The approach in general uses the photoionization rate from each level,
   *  which is easier to find than recombination rates.  
   */ 

void calc_rad_rec_cont(struct DEBUG_FILES *debug, 
		       int Z, int rmJ, double abund, double this_ion_pop, 
		       struct ELEMENT_ION_STATE *pstate,
		       struct ELEMENT_ION_STATE *state,
		       double Te, double min_emission,
		       double *LevelRecombRate, struct BIN_SPEC *spectrum) {


  
  int nlev, sig_type=NO_PHOT;
  int ne; /*!< Number of electrons in recombined ion */ 
  int iLev, i, iBin;
  double I_e=0.0; /*!< Ionization energy of level, in keV */
  double lev_deg=0.0, parent_lev_deg=1.0, g_ratio;
  double rr_lev_rate, tot_rec_rate=0.0;
  double *emission=NULL;
  double sigma_coeff[MAX_SIG];
  struct COMPRESS_CONT *cs_ptr;

  if (spectrum != NULL) {
    emission = (double *) malloc_safe(spectrum->Nbin*sizeof(double));
    for (iBin=0;iBin<spectrum->Nbin;iBin++) emission[iBin] = 0.0;
  }

  nlev = state->ion_inf->nlev;

  ne = Z - rmJ + 1;

  for (iLev=0;iLev<nlev;iLev++) {
    switch (state->lev_dat[iLev].phottype) {
    case (NO_PHOT):
      break;
    case (HYDROGENIC):
      if (ne == 1) { /* Hydrogenic; go up to n = 5 */
	/* Calculate ionization energy for level */
	I_e = RYDBERG*Z*Z/(state->lev_dat[iLev].N*state->lev_dat[iLev].N);
	lev_deg = state->lev_dat[iLev].deg;
	parent_lev_deg = 1.0;
	sig_type = HYDROGENIC; 
	sigma_coeff[0] = state->lev_dat[iLev].N;
	sigma_coeff[1] = state->lev_dat[iLev].L;
	for (i=2;i<MAX_SIG;i++) sigma_coeff[i] = 0.0;
      } 
      break;
    case (CLARK):
      I_e = state->lev_dat[iLev].sig[1];  /* in keV */
      lev_deg = state->lev_dat[iLev].deg;  
      if ((pstate != NULL)&&(pstate->ion_inf->nlev > 0)&&
	  (pstate->lev_dat != NULL)) 
	parent_lev_deg = pstate->lev_dat[0].deg; 
     sig_type = CLARK;
      for (i=0;i<MAX_SIG;i++) sigma_coeff[i] = state->lev_dat[iLev].sig[i];
      break;
    case (VERNER):  /* Ground state recombination */
      I_e = state->lev_dat[iLev].sig[0];  /* in keV */
      lev_deg = state->lev_dat[iLev].deg;
      if ((pstate != NULL)&&(pstate->ion_inf->nlev >0)&&
	  (pstate->lev_dat != NULL)) 
	parent_lev_deg = pstate->lev_dat[0].deg; 
      sig_type = VERNER;
      for (i=0;i<MAX_SIG;i++) sigma_coeff[i] = state->lev_dat[iLev].sig[i];
      sigma_coeff[6] = state->lev_dat[iLev].L;
      break;
    default:
      errmess("calc_rad_rec_cont","Illegal datatype %d",
	      state->lev_dat[iLev].phottype);
      break;
    }
    rr_lev_rate = 0.0;
    g_ratio = lev_deg/parent_lev_deg;
    if (state->lev_dat[iLev].phottype != NO_PHOT) {
      /* Calculate the actual recombination */
      calc_rrc(Z, rmJ, abund, this_ion_pop, I_e, g_ratio,
	       sig_type, sigma_coeff, Te, min_emission,
	       spectrum, emission, &rr_lev_rate);
      LevelRecombRate[iLev] = rr_lev_rate; 

      tot_rec_rate += rr_lev_rate;
      if (debug != NULL) {
	if (debug->do_qrec && debug->ascii) {
	  fprintf(debug->qrec.ascii,"%d %d %g %d %e %e\n",Z, rmJ, Te, iLev, 
		  rr_lev_rate,tot_rec_rate); 
	}
	if (debug->do_qrec && debug->fits) {
	  fd_write_row(&debug->qrec.fits, Z, rmJ, Te, iLev, rr_lev_rate,
		       tot_rec_rate, 0.0); 
	}
      }
    }
  }

  /* Calculate compressed spectrum, put into spectrum->cs.  Note that since */
  /* the RRC emissivity is proportional to the parent ion, we put the       */
  /* resulting emissivity in Z, rmJ+1, not Z, rmJ                           */
  
  if (spectrum != NULL) {
    cs_ptr = spectrum->cs;
    while ((cs_ptr != NULL) && ((cs_ptr->Z != Z) || (cs_ptr->rmJ != rmJ+1))) 
      cs_ptr = cs_ptr->next;
    if (cs_ptr == NULL) { 
      message("calc_rad_rec_cont",
	      "Output ion spectrum for %s not found, skipping.",
	      roman_element(Z,rmJ));
    } else {
      for (iBin=0;iBin<spectrum->Nbin;iBin++) 
	cs_ptr->bin_real[iBin] += emission[iBin];
      compress_energy_spectrum(spectrum->Nbin, spectrum->bin_edge, emission, 
			       ERROR_RRC, "RRC", &(cs_ptr->Nrrc), 
			       &(cs_ptr->E_rrc), &(cs_ptr->rrc));
    }
  }
  if (debug != NULL) {
    if (debug->do_qrec && debug->ascii) fflush(debug->qrec.ascii);
  }

  if (spectrum != NULL) free(emission);

}
