#include <stdlib.h>
#include <stdio.h>

#include "apec.h"
#include "messages.h"

/* calc_recomb_rate():
   In process of doing radiative recombination...
   function will eventually calculate the net recombination rate 
   to each level, and the total recombination rate for the ion.
   Will call separate functions to get radiative,
   dielectronic, and 3-body recomb rates.
   Add qrecomb_total to level_rates in apec.h and remove from calling
   function in atomic. Distribute it according to the fraction
   of dielectronic recombination to each level.
*/

void calc_recomb_rate(double Te, double Ne, struct DEBUG_FILES *debug, 
		      struct PARAMETERS *params,
		      struct ELEMENT_ION_STATE *state, double *recomb_rate) {

  int nlev, Z;
  double alpharr, alphadr;
  double a, b, T0, T1;
  double sqrtT_T0, sqrtT_T1, expT_T0, expT_T1;
  
  nlev = state->ion_inf->nlev;
  Z = state->ion_inf->Z;
  alpharr = 0.0;
  alphadr = 0.0;

  if (state->ion_inf->ns > 0) {
    switch(state->ionrec_dat[0].rr_type) {
    case (0):
      message("calc_recomb_rate","No RR rates coefficient available.");
      alpharr = 0.0;
      break;
    case (1):
      a = state->ioniz_dat[0].rr_c[0];
      b = state->ioniz_dat[0].rr_c[1];
      T0 = state->ioniz_dat[0].rr_c[2];
      T1 = state->ioniz_dat[0].rr_c[3];
      sqrtT_T0 = sqrt(Te/T0);
      sqrtT_T1 = sqrt(Te/T1);
      /* From Verner & Ferland, ApJS, 103, 467 */
      alpharr = a / (sqrtT_T0*pow(1+sqrtT_T0,1-b)*pow(1+sqrtT_T1,1+b));      
      break;
    default:
      errmess("calc_recomb_rate","Illegal recombination rate coefficient");
      break;
    }
    
    /* do the dielectronic rate here */
    /* From Burgess formula?, for hydrogenic */
  
    switch(state->ioniz_dat[0].dr_type) {
    case (0):
      message("calc_recomb_rate","No DR rates coefficient available.");
      alphadr = 0.0;
      break;
    case (1):
      a = state->ioniz_dat[0].dr_c[0];
      b = state->ioniz_dat[0].dr_c[1];
      T0 = state->ioniz_dat[0].dr_c[2];
      T1 = state->ioniz_dat[0].dr_c[3];
      expT_T0 = exp(-T0/Te);
      expT_T1 = exp(-T1/Te);
      /* From S. M. V. Aldrovandi & D. Pequignot (1973, A&A, 25, 137),
	 J. M. Shull & M. Van Steenberg (1982, ApJS, 48, 95),
	 M. Arnaud & R. Rothenflug (1985, A&AS, 60, 425). */
      alphadr = a*pow(Te,-1.5)*expT_T0*(1+b*expT_T1);
      break;
    default:
      errmess("calc_recomb_rate","Illegal recombination rate coefficient");
      break;
    }
  }
  
  *recomb_rate = alpharr + alphadr; /* plus eventually other
						     recomb terms */
  
  if (debug->do_qrec && debug->ascii) {
    fprintf(debug->qrec.ascii,"total recombination rate = %g\n", *recomb_rate);
  }
  if (debug->do_qrec & debug->fits) {
    fd_write_row(&debug->qrec.fits, 0, 0, 0.0, 0, 0.0, 0.0, *recomb_rate);
  }
}

