#include <stdlib.h>
#include <math.h>

#include "apec.h"
#include "messages.h"

/*! \file calc_satellite.c */

void write_debug(struct DEBUG_GENERIC *debug, int ascii, int fits, 
		 char* fmt, ...);



/*! \brief Calculate dielectronic satellite line emission and level population.
 *  \param debug Debugging output files [optional; set to NULL to ignore]
 *  \param debug 
 *  \param abund Abundance of element Z
 *  \param this_ion_pop Ion population of recombined ion (unitless)
 *  \param pstate Atomic data for parent (recombining)ion 
 *  \param state Atomic data for recombined ion
 *  \param Te electron temperature, in Kelvin 
 *  \param Ne electron density, in cm^-3
 *  \param sat_recomb_rate Output vector of total satellite recombination rate to each level 
 *  \param line line output structure [optional; set to NULL to ignore]
 * 
 * This value for SAF_COEFF is from Kato et al 1997, ADNDT 67, 226. 
 * Basically, the intensity of a satellite line is given by 
 * I= 4 pi^{3/2} a_0^{3/2} (kT/Rydberg)^{-3/2} Q_exc/g_l exp(-E_{exc}/(kT)) 
 * where a_0 is the Bohr radius, g_l is the statistical weight of the ground 
 * state of the parent ion, ie the ion state _before_ the satellite 
 * electron is attached.  Q_exc is 
 * defined as g_s A_a A_r / (sum A_a + sum A_r), where the sum is taken 
 * over all possible decay states and g_s is the stat weight of the doubly- 
 * excited state.  The upshot of all this is that 4 pi^{3/2} a_0^{3/2} is 
 * equal to 3.30e-24, and so that is the value here. 
 */

#define SAF_COEFF 3.30e-24  

void calc_satellite(struct DEBUG_FILES *debug,
		    double abund, double Ion_pop,
		    struct ELEMENT_ION_STATE *pstate,
		    struct ELEMENT_ION_STATE *dstate,		    
		    double Te, double Ne, 
		    double sat_recomb_rate[], 
		    struct EMISSION_LINES **line) {

  /* Ion_pop = unitless population of parent ion state */
  /* lambda = wavelength (A) of line */
  /* epsilon = atomic factor [photons cm+3 s-1] */
  /* Intensity (obs at earth) [photons cm-2 s-1]
     = epsilon * Emission Measure/ (4 pi R^2) */
  /* Plambda = Line emission in [ergs cm^3 s-1] */

  int rmJ, Z;
  int ll, lu, gl;
  long max_satel, nt, nlev;

  double lambda, lambda_err, epsilon, Plambda, E_excite;
  double q_exc=0.0;
  double kT;


  struct EMISSION_LINES *lptr;
  
  kT = KBOLTZ*Te;
  Z = pstate->ion_inf->Z;
  rmJ = pstate->ion_inf->rmJ;
  max_satel = pstate->ion_inf->nsatel;

  lptr = NULL;
  if (line != NULL) lptr = *line;

  nlev = dstate->ion_inf->nlev;
 
  /* Calculate the line emission  */

  for (nt = 0; nt < max_satel; nt++) {
    epsilon = 0.;
    ll = pstate->satellite_dat[nt].lev_lower;
    lu = pstate->satellite_dat[nt].lev_upper;
    lambda = pstate->satellite_dat[nt].lambda;
    lambda_err = pstate->satellite_dat[nt].lambda_err;
    E_excite = pstate->satellite_dat[nt].E_excite; /* in keV */
    switch(pstate->satellite_dat[nt].type) {
    case (ROMANIK):
      q_exc = pstate->satellite_dat[nt].q_exc;
      epsilon = q_exc*exp(-E_excite/kT)/(Te*sqrt(Te));/* phot cm^3 / s */
      sat_recomb_rate[ll-1] += epsilon;
      break;
    case (SAFRONOVA):
      gl = pstate->lev_dat[0].deg;  /* Ground state of parent ion */
      q_exc = pstate->satellite_dat[nt].q_exc;
      epsilon = SAF_COEFF*pow(RYDBERG/kT,1.5)*(q_exc/gl)*exp(-E_excite/kT);
      sat_recomb_rate[ll-1] += epsilon;
      break;
    default:
      errmess("calc_satellite","Illegal DR satellite type %d",
	      pstate->satellite_dat[nt].type);
      break;
    }
    
    if (debug != NULL) {
      if (debug->do_satel) {
	write_debug(&debug->satel, debug->ascii, debug->fits,
		    "%3d %3d %5d %5d %10g %10g %10g %10g %10g\n",
		    Z, rmJ, lu, ll, Te, lambda, E_excite, q_exc, epsilon );
      }
    }

    if ((line != NULL) && (lambda > 0)) {
      if (lptr == NULL) { 
	*line = (struct EMISSION_LINES *)
	  malloc_safe(sizeof(struct EMISSION_LINES));
	lptr = *line;
      } else { /* Find last link & make new one */
	while (lptr->next != NULL) lptr = lptr->next;
	lptr->next = (struct EMISSION_LINES *)
	  malloc_safe(sizeof(struct EMISSION_LINES));
	lptr = lptr->next;
      }
      lptr->next = NULL;

      epsilon *= abund*Ion_pop;
      Plambda = epsilon * (HC_IN_ERG_A/lambda);  /*  erg cm^3 / s */
    
      lptr->epsilon = epsilon; /* phot cm^3 / s */
      lptr->Plambda = Plambda;  /*  erg cm^3 / s */
      lptr->lambda  = lambda;  /* Angstroms */
      lptr->lambda_err=lambda_err;  /* Angstroms */
      lptr->Z = Z;
      lptr->rmJ = rmJ;
      lptr->ul= lu;
      lptr->ll= ll;
      lptr->type = SATELLITE_LINE;
      if (debug != NULL) {
	if ((debug->do_emiss) && (epsilon != 0)) {
	  write_debug(&debug->emiss, debug->ascii, debug->fits,
		      "%2d %2d %7.2f %7.2f %2d %2d %8.2f %8.2e %8.2e\n",
		      Z,rmJ,log10(Ne),log10(Te),lu,ll,lambda,epsilon,Plambda);
	}
      }
    } else {
      /* message("calc_emission","Bad line wavelength") */
    }
  }

  if (debug != NULL) {
    if (debug->do_satel && debug->ascii) fflush(debug->satel.ascii);
    if (debug->do_emiss && debug->ascii) fflush(debug->emiss.ascii);
  }
}
