#include <stdlib.h>
#include <stdio.h>
#include <fitsio.h>

#include "apec.h"
#include "messages.h"

void output_comp_emission(struct PARAMETERS *params, double T, double Ne, 
			  struct BIN_SPEC *spectrum);

void write_fits_spectral_table(struct PARAMETERS *params, 
			       fitsfile *fptr_spectral, int iTemp, 
			       int iDens, struct BIN_SPEC *spectrum);
void fits_spec_write_continuum(fitsfile *fptr, double T, double n_e,
			       int nbin, double Ebin_low[], 
			       double Ebin_high[], double pseudo_cont[], 
			       double pseudo_cont_err[],
			       double brems[], double brems_err[],
			       double twoph[], double twoph_err[],
			       double radrec[], double radrec_err[]);
void fits_spec_write_lines(fitsfile *fptr, double T, double n_e, int nlines,
                           double lambda[],  double lambda_err[], 
                           double epsilon[], double epsilon_err[], 
                           int el_Z[], int rmJ[],
                           int UpperLev[], int LowerLev[]);
void fits_spec_write_coco(fitsfile *fptr, int N_Z, int list_Z[],
			  int list_rmJ[], double T, double Ne, int nbin, 
			  double E_low[], double E_high[],
			  double pseu_arr[],double cont_arr[]);

void fits_output(struct PARAMETERS *params, double Te, double Ne, int iTemp,
		 int iDens, struct EMISSION_LINES **line, 
		 struct BIN_SPEC *spectrum) {

  int iZ, Z, N_Z, nbin, iBin, iLine, nlines=0;
  double *E_low, *E_high, *real_cont, *pseudo_cont;
  double *pseudo, *brems, *twoph, *rrc;
  double *lambda=NULL, *lambda_err=NULL, *epsilon=NULL;
  int *el_Z=NULL, *rm_J=NULL, *UpperLev=NULL, *LowerLev=NULL;
  struct EMISSION_LINES *lptr;

  /***************************************/
  /*  Write out the line emissivity file */
  /***************************************/

  if (params->do_line) {
    lptr = *line;
    while (lptr != NULL) {
      if (lptr->epsilon > params->min_epsilon) nlines++;
      lptr = lptr->next;
    }
    if (nlines != 0) {
      lambda    = (double *) malloc_safe(nlines*sizeof(double));
      lambda_err= (double *) malloc_safe(nlines*sizeof(double));
      epsilon   = (double *) malloc_safe(nlines*sizeof(double)); 
      el_Z      = (int *) malloc_safe(nlines*sizeof(int));
      rm_J      = (int *) malloc_safe(nlines*sizeof(int));
      UpperLev  = (int *) malloc_safe(nlines*sizeof(int));
      LowerLev  = (int *) malloc_safe(nlines*sizeof(int));
    }
    iLine = 0;
    lptr = *line;
    while (lptr != NULL) {
      if (lptr->epsilon > params->min_epsilon) {
	lambda[iLine]     = lptr->lambda;
	lambda_err[iLine] = lptr->lambda_err;
	epsilon[iLine]    = lptr->epsilon;
	el_Z[iLine]       = lptr->Z;
	rm_J[iLine]       = lptr->rmJ;
	UpperLev[iLine]   = lptr->ul;
	LowerLev[iLine]   = lptr->ll;
	iLine++;
      }
      lptr = lptr->next;
    }
    
    fits_spec_write_lines(params->fptr_line, Te, Ne, nlines, lambda, 
			  lambda_err, epsilon, NULL, el_Z, rm_J, 
			  UpperLev, LowerLev);
    if (nlines != 0) {
      free(lambda);
      free(lambda_err);
      free(epsilon);
      free(el_Z);
      free(rm_J);
      free(UpperLev);
      free(LowerLev);
    }
  }

  /***********************************************/
  /*  Write out the uncompressed continuum file. */
  /***********************************************/

  if (params->do_bincont) {
    nbin = spectrum->Nbin;
    N_Z = params->N_Elements;
    E_low  = (double *) malloc_safe(nbin*sizeof(double));
    E_high = (double *) malloc_safe(nbin*sizeof(double));
    pseudo = (double *) malloc_safe(nbin*sizeof(double));
    brems  = (double *) malloc_safe(nbin*sizeof(double));
    twoph  = (double *) malloc_safe(nbin*sizeof(double));
    rrc    = (double *) malloc_safe(nbin*sizeof(double));
    
    for (iBin=0;iBin<nbin;iBin++) {
      E_low[iBin] = (spectrum->bin_edge)[iBin];
      E_high[iBin]= (spectrum->bin_edge)[iBin+1];
      for (iZ=0;iZ<N_Z;iZ++) {
	Z = (params->list_Z)[iZ];
	pseudo[iBin]= (spectrum->pseudo_emiss[Z])[iBin];
	brems[iBin] = (spectrum->brems_emiss[Z])[iBin];
	twoph[iBin] = (spectrum->twoph_emiss[Z])[iBin];
	rrc[iBin]   = (spectrum->rrc_emiss[Z])[iBin];
      }
    }
      
    fits_spec_write_continuum(params->fptr_bincont, Te, Ne, nbin,
			      E_low, E_high, pseudo, NULL, brems, NULL, 
			      twoph, NULL,rrc, NULL);
    free(E_low);
    free(E_high);
    free(pseudo);
    free(brems);
    free(twoph);
    free(rrc);
  }

  /********************************************************/
  /*  Write out the per element compressed continuum file */
  /********************************************************/
  
  if (params->do_coco) {
    N_Z = params->N_Elements;
    nbin = spectrum->Nbin;
    E_low      = (double *) malloc_safe(N_Z*nbin*sizeof(double));
    E_high     = (double *) malloc_safe(N_Z*nbin*sizeof(double));
    real_cont  = (double *) malloc_safe(N_Z*nbin*sizeof(double));
    pseudo_cont= (double *) malloc_safe(N_Z*nbin*sizeof(double));
    for (iZ=0;iZ<N_Z;iZ++) {
      Z = (params->list_Z)[iZ];
      E_high[iZ + N_Z*0] = -1.0;
      for (iBin=0;iBin<nbin;iBin++) {
	E_low[iZ + N_Z*iBin]  = (spectrum->bin_edge)[iBin];
	E_high[iZ + N_Z*iBin] = (spectrum->bin_edge)[iBin+1];
	real_cont[iZ + N_Z*iBin] = (spectrum->brems_emiss[Z])[iBin] + 
	  (spectrum->twoph_emiss[Z])[iBin] + (spectrum->rrc_emiss[Z])[iBin];
	pseudo_cont[iZ + N_Z*iBin] = (spectrum->pseudo_emiss[Z])[iBin];
      }
    }
    
    fits_spec_write_coco(params->fptr_coco, N_Z, params->list_Z, NULL,Te,Ne,
			 nbin,E_low, E_high, pseudo_cont, real_cont);
    free(E_low);
    free(E_high);
    free(real_cont);
    free(pseudo_cont);
  }

  /*****************************************************/
  /*  Write out the per-ion compressed continuum file. */  
  /*****************************************************/
  
  if (params->do_comp) {
    output_comp_emission(params, Te, Ne, spectrum);
  }

  /***********************************************/
  /*  Write out an XSPEC v10.0 table model file. */
  /***********************************************/
  
  if (params->do_xspectable) {
    write_fits_spectral_table(params, params->fptr_xspectable, iTemp, iDens, 
			      spectrum);
  }
  
}
