#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <fitsio.h>

#include "messages.h"
#include "apec.h"
#include "datatypes.h"

unsigned long read_fits_level_data(int require_checksum,
				   fitsfile *fptr, int *Nvalues,
				   struct LEVEL_DATA **);
unsigned long read_fits_radiation_data(int require_checksum,
				       fitsfile *fptr, long *Nvalues,
				       struct LINERAD_DATA **);
unsigned long read_fits_collision_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				       struct COLLTRNS_DATA **);
unsigned long read_fits_satellite_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				       struct SATELLITE_DATA **);
unsigned long read_fits_emission_data(int require_checksum,
				      fitsfile *fptr, int *Nvalues,
				      struct EMISSION_DATA **);
unsigned long check_fits_file(fitsfile *fptr);
void fits_printerror(char *process, int status);

/* ******************************************************************* */
/* Note: These routines are chock-full of assumptions about how long   */
/* fields are, what datatype they are, etc.  There's no                */
/* way to really avoid this, since the data structures cannot be       */
/* defined on the fly.  So this is the big "link" routine between      */
/* the fits format files and the internal data structures.  Everything */
/* really ugly is done here, and every time the version changes, this  */
/* routine will need to change.                                        */
/* ******************************************************************* */

unsigned long read_fits_level_data(int require_checksum,
				   fitsfile *fptr, int *Nvalues,
				   struct LEVEL_DATA **lev_dat_ptr) {


  /* Input assumptions: 
     fptr is a fits file ptr, indexed to the correct HDU. 
     lev_dat is an LEVEL_DATA pointer for the data in that HDU.
     Output requirements:
     lev_dat should be loaded up with all the appropriate data.  
   */
  
  int Nrows, iCol, iRow, anynul, i;
  int status=0;
  float var_float;
  float arr_float[MAX_SIG];
  char *colnames[] = {"Energy","E_Error","Lev_Deg","N_quan","L_quan",
		       "S_quan","Phot_Type","Phot_Par","Elec_Config"};
  int colnum[9]; /* 9 names listed above */
  int Ncols = 9;
  char comment[MAXSTRLEN];
  struct LEVEL_DATA *lev_dat;
  unsigned long datasum = 0;
  char buf[MAXSTRLEN] = "";
  char * buf_ptr = buf; 
  char ** buf_ptr_ptr = &buf_ptr;

  /* First, check that the is file OK and return the checksum and datasum */
  if (require_checksum) datasum = check_fits_file(fptr);

  /* Figure out how many values we have to load. */
  fits_read_key(fptr, TINT, "NAXIS2", &Nrows, comment, &status);
  
  if (Nrows == 0) { /* No actual data here. */
    *lev_dat_ptr = NULL;
    *Nvalues = Nrows; 
    return datasum;
  } else { /* Make space for the data */
    *lev_dat_ptr = (struct LEVEL_DATA *)
      malloc_safe(Nrows*sizeof(struct LEVEL_DATA));
    lev_dat = *lev_dat_ptr;
  }
  
  /* Get the column numbers */
  
  for(iCol=0;iCol<Ncols;iCol++) {
    fits_get_colnum(fptr, CASEINSEN, colnames[iCol], &(colnum[iCol]), &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s column number",
	       colnames[iCol]);
    }
  }
  
  /* Now, read in the data. */
  for (iRow=1;iRow<Nrows+1;iRow++) {
    lev_dat[iRow-1].lev = iRow;

    fits_read_col(fptr, TFLOAT, colnum[0], iRow, 1, 1, 0, &var_float, &anynul,
		  &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[0]);
    } else {
      lev_dat[iRow-1].energy = (double) var_float;
    }
    
    fits_read_col(fptr,TINT,colnum[2],iRow,1,1,0,&(lev_dat[iRow-1].deg),
		  &anynul,&status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[2]);
    }

    fits_read_col(fptr,TINT,colnum[3],iRow,1,1,0,&(lev_dat[iRow-1].N),
		  &anynul,&status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[3]);
    }
    fits_read_col(fptr,TINT,colnum[4],iRow,1,1,0,&(lev_dat[iRow-1].L),
		  &anynul,&status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[4]);
    }
    fits_read_col(fptr,TFLOAT,colnum[5],iRow,1,1,0,&(lev_dat[iRow-1].S),
		  &anynul,&status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[5]);
    }

    fits_read_col(fptr,TINT,colnum[6],iRow,1,1,0,&(lev_dat[iRow-1].phottype),
		  &anynul,&status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[6]);
    }

    fits_read_col(fptr,TFLOAT,colnum[7],iRow,1,MAX_SIG,0,arr_float,
		  &anynul,&status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[7]);
    }
    for (i=0;i<MAX_SIG;i++) lev_dat[iRow-1].sig[i] = arr_float[i];

    *lev_dat[iRow-1].config = '\0';
    *buf = '\0';
    fits_read_col(fptr, TSTRING, colnum[8], iRow, 1, 1, 0, 
		  buf_ptr_ptr, &anynul, &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[8]);
      status = 0;
    } else {
      strncat(lev_dat[iRow-1].config, buf, MAXSTRLEN);
    }
  }

  *Nvalues = Nrows;
  return datasum; 
}

unsigned long read_fits_radiation_data(int require_checksum,
				       fitsfile *fptr, long *Nvalues,
				       struct LINERAD_DATA **linerad_dat_ptr){

  /* Input assumptions: 
     fptr is a fits file ptr, indexed to the correct HDU. 
     linerad_dat is an LINERAD_DATA pointer for the data in that HDU.
     Output requirements:
     linerad_dat should be loaded up with all the appropriate data.  
   */
  
  int Nrows, iCol, iRow, anynul;
  int status=0;
  float var_float, fnul_val;
  char *colnames[7] = {"Upper_Lev","Lower_Lev","Wavelen","Wave_Obs",
		       "Wave_Err","Einstein_A","Ein_A_Err"};
  int colnum[7]; /* 7 names listed above */
  int Ncols = 7;
  char comment[MAXSTRLEN];
  struct LINERAD_DATA *linerad_dat;
  unsigned long datasum = 0;

  /* First, check that the is file OK and return the checksum and datasum */
  if (require_checksum) datasum = check_fits_file(fptr);
  
  /* Figure out how many values we have to load. */
  fits_read_key(fptr, TINT, "NAXIS2", &Nrows, comment, &status);

  if (Nrows == 0) { /* No actual data here. */
    *linerad_dat_ptr = NULL;
    *Nvalues = (long) Nrows; 
    return datasum;
  } else { /* Make space for the data */
    *linerad_dat_ptr = (struct LINERAD_DATA *) 
      malloc_safe(Nrows*sizeof(struct LINERAD_DATA));
    linerad_dat = *linerad_dat_ptr;
  }

  /* Get the column numbers */
  
  for(iCol=0;iCol<Ncols;iCol++) {
    fits_get_colnum(fptr, CASEINSEN, colnames[iCol], &(colnum[iCol]), &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s column number",
	       colnames[iCol]);
    }
  }

  /* Now, read in the data. */
  for (iRow=1;iRow<Nrows+1;iRow++) {
    fits_read_col(fptr, TINT, colnum[0], iRow, 1, 1, 0,
		  &(linerad_dat[iRow-1].lev_upper), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[0]);
    }

    fits_read_col(fptr, TINT, colnum[1], iRow, 1, 1, 0,
		  &(linerad_dat[iRow-1].lev_lower), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[1]);
    }

    fits_read_col(fptr, TFLOAT, colnum[2], iRow, 1, 1, 0,&var_float, &anynul,
		  &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[2]);
    } else {
      linerad_dat[iRow-1].lambda = (double) var_float;
    }

    /* Now, if the lambda_obs column is not NULL, use this value instead. */
    /* This covers both version 1.0 files (lambda_obs = lambda by default) */
    /* and version 1.1 files (lambda_obs = NULL unless a value exists) */

    fnul_val = -1.0;
    fits_read_col(fptr, TFLOAT, colnum[3], iRow, 1, 1, &fnul_val, &var_float, 
		  &anynul, &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[3]);
    } else {
      if (anynul == FALSE) linerad_dat[iRow-1].lambda = (double) var_float;
    }

    fnul_val = -1.0;
    fits_read_col(fptr, TFLOAT, colnum[4], iRow, 1, 1, &fnul_val, &var_float, 
		  &anynul, &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[4]);
    } else {
      if (anynul == FALSE) {
	linerad_dat[iRow-1].lambda_err = (double) var_float; 
      } else {
	linerad_dat[iRow-1].lambda_err = (double) fnul_val; 
      }
    }

    fits_read_col(fptr, TFLOAT, colnum[5], iRow, 1, 1, 0,&var_float, &anynul,
		  &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[5]);
    } else {
      linerad_dat[iRow-1].A = (double) var_float;
    }
  }
  *Nvalues = (long) Nrows;
  return datasum;
}

unsigned long read_fits_collision_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				       struct COLLTRNS_DATA 
				       **colltrns_dat_ptr) {

  /* Input assumptions: 
     fptr is a fits file ptr, indexed to the correct HDU. 
     colltrns_dat is an COLLTRNS_DATA pointer for the data in that HDU.
     Output requirements:
     colltrns_dat should be loaded up with all the appropriate data.  
   */
  
  int Nrows, iCol, iRow, i, anynul;
  int status=0;
  float var_float;
  float arr_float[MAX_UPS];
  char *colnames[7];
  char *colnames11[7] = {"Lower_Lev","Upper_Lev","Coeff_Type","Min_Temp",
			  "Max_Temp","Temperature","EffCollStrPar"};
  char *colnames10[7] = {"Lower_Lev","Upper_Lev","Coeff_Type","Min_Temp",
			     "Max_Temp","Temperature","Coeff_OM"};
  int colnum[7]; /* 7 names listed above */
  int Ncols = 7;
  char comment[MAXSTRLEN], version[MAXSTRLEN], filename[MAXSTRLEN];
  struct COLLTRNS_DATA *colltrns_dat;
  unsigned long datasum = 0;

  /* First, check that the is file OK and return the checksum and datasum */
  if (require_checksum) datasum = check_fits_file(fptr);

  /* Figure out how many values we have to load. */
  fits_read_key(fptr, TINT, "NAXIS2", &Nrows, comment, &status);

  if (Nrows == 0) { /* No actual data here. */
    *colltrns_dat_ptr = NULL;
    *Nvalues = Nrows; 
    return datasum;
  } else { /* Make space for the data */
    *colltrns_dat_ptr = (struct COLLTRNS_DATA *)
      malloc_safe(Nrows*sizeof(struct COLLTRNS_DATA));
    colltrns_dat = *colltrns_dat_ptr;
  }

  /* Get the column numbers */

  /* What version to read? */
  fits_read_key(fptr, TSTRING, "HDUVERS1", version, comment, &status);
  if (strncmp(version,"1.0",3) == 0) {
    for(iCol=0;iCol<Ncols;iCol++) colnames[iCol] = colnames10[iCol];
  } else if (strncmp(version,"1.1",3) == 0) {
    for(iCol=0;iCol<Ncols;iCol++) colnames[iCol] = colnames11[iCol];
  } else {
    if (fits_file_name(fptr, filename, &status)) {
      fits_printerror("read_fits_datafiles", status);
    }
    errmess("read_fits_datafiles","Unknown version %s in file %s",version,
	    filename);
  }
  
  for(iCol=0;iCol<Ncols;iCol++) {
    fits_get_colnum(fptr, CASEINSEN, colnames[iCol], &(colnum[iCol]), &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s column number",
	       colnames[iCol]);
    }
  }

  /* Now, read in the data. */
  for (iRow=1;iRow<Nrows+1;iRow++) {
    fits_read_col(fptr, TINT, colnum[0], iRow, 1, 1, 0,
		  &(colltrns_dat[iRow-1].lev_lower), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[0]);
    }

    fits_read_col(fptr, TINT, colnum[1], iRow, 1, 1, 0,
		  &(colltrns_dat[iRow-1].lev_upper), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[1]);
    }

    fits_read_col(fptr, TINT, colnum[2], iRow, 1, 1, 0,
		  &(colltrns_dat[iRow-1].coll_type), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[2]);
    }

    fits_read_col(fptr, TFLOAT, colnum[3], iRow, 1, 1, 0, &(var_float), 
		  &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[3]);
    }
    colltrns_dat[iRow-1].min_T = (double) var_float;

    fits_read_col(fptr, TFLOAT, colnum[4], iRow, 1, 1, 0, &(var_float), 
		  &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[4]);
    }
    colltrns_dat[iRow-1].max_T = (double) var_float;

    if (fits_read_col(fptr, TFLOAT, colnum[5], iRow, 1, MAX_UPS, 0, arr_float,
		      &anynul, &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[5]);
    } else {
      for (i=0;i<MAX_UPS;i++) colltrns_dat[iRow-1].T[i]=(double) arr_float[i];
    }

    if (fits_read_col(fptr, TFLOAT, colnum[6], iRow, 1, MAX_UPS, 0, arr_float,
		      &anynul, &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[6]);
    } else {
      for (i=0;i<MAX_UPS;i++) colltrns_dat[iRow-1].om[i]=(double) arr_float[i];
    }
  }
  *Nvalues = Nrows; 
  return datasum;
}


unsigned long read_fits_satellite_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				       struct SATELLITE_DATA 
				       **satellite_dat_ptr) {

  /* Input assumptions: 
     fptr is a fits file ptr, indexed to the correct HDU. 
     satellite_dat is an SATELLITE_DATA pointer for the data in that HDU.
     Output requirements:
     satellite_dat should be loaded up with all the appropriate data.  
   */
  
  int Nrows, iCol, iRow, iArr, anynul;
  int status=0;
  float var_float, fnul_val;
  float arr_float[DR_PAR];
  char *colnames[9] = {"Upper_Lev","Lower_Lev","Wavelen","Wave_Obs","Wave_Err",
		       "DR_Type","E_Excite","SatelInt","Params"};
  int colnum[9]; /* 9 names listed above */
  int Ncols = 9;
  char comment[MAXSTRLEN];
  struct SATELLITE_DATA *satellite_dat;
  unsigned long datasum = 0;

  /* First, check that the is file OK and return the checksum and datasum */
  if (require_checksum) datasum = check_fits_file(fptr);

  /* Figure out how many values we have to load. */
  fits_read_key(fptr, TINT, "NAXIS2", &Nrows, comment, &status);

  if (Nrows == 0) { /* No actual data here. */
    *satellite_dat_ptr = NULL;
    *Nvalues = Nrows; 
    return datasum;
  } else { /* Make space for the data */
    *satellite_dat_ptr = (struct SATELLITE_DATA *) 
      malloc_safe(Nrows*sizeof(struct SATELLITE_DATA));
    satellite_dat = *satellite_dat_ptr;
  }

  /* Get the column numbers */
  
  for(iCol=0;iCol<Ncols;iCol++) {
    fits_get_colnum(fptr, CASEINSEN, colnames[iCol], &(colnum[iCol]), &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s column number",
	       colnames[iCol]);
    }
  }

  /* Now, read in the data. */
  for (iRow=1;iRow<Nrows+1;iRow++) {
    fits_read_col(fptr, TINT, colnum[0], iRow, 1, 1, 0,
		  &(satellite_dat[iRow-1].lev_upper), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[0]);
    }

    fits_read_col(fptr, TINT, colnum[1], iRow, 1, 1, 0,
		  &(satellite_dat[iRow-1].lev_lower), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[1]);
    }

    fits_read_col(fptr, TFLOAT, colnum[2], iRow, 1, 1, 0,&var_float, &anynul,
		  &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[2]);
    } else {
      satellite_dat[iRow-1].lambda = (double) var_float;
    }
    
    /* Now, if the lambda_obs column is not NULL, use this value instead. */
    /* This covers both version 1.0 files (lambda_obs = lambda by default) */
    /* and version 1.1 files (lambda_obs = NULL unless a value exists) */

    fnul_val = -1.0;
    fits_read_col(fptr, TFLOAT, colnum[3], iRow, 1, 1, &fnul_val, &var_float, 
		  &anynul, &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[3]);
    } else {
      if (anynul == FALSE) satellite_dat[iRow-1].lambda = (double) var_float;
    }

    fnul_val = -1.0;
    fits_read_col(fptr, TFLOAT, colnum[4], iRow, 1, 1, &fnul_val, &var_float, 
		  &anynul, &status);
    if (status != 0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[4]);
    } else {
      if (anynul == FALSE) {
	satellite_dat[iRow-1].lambda_err = (double) var_float; 
      } else {
	satellite_dat[iRow-1].lambda_err = (double) fnul_val; 
      }
    }


    if (fits_read_col(fptr, TINT, colnum[5], iRow, 1, 1, 0,
		      &(satellite_dat[iRow-1].type), &anynul, &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[5]);
    }

    if (fits_read_col(fptr,TFLOAT,colnum[6],iRow,1,1,0,&var_float,&anynul,
		      &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[6]);
    } else {
      satellite_dat[iRow-1].E_excite = (double) var_float;
    }

    if (fits_read_col(fptr,TFLOAT,colnum[7],iRow,1,1,0,&var_float,&anynul,
		      &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[7]);
    } else {
      satellite_dat[iRow-1].q_exc = (double) var_float;
    }
    
    if (fits_read_col(fptr,TFLOAT,colnum[8],iRow,1,DR_PAR,0,arr_float,&anynul,
		      &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[8]);
    } else {
      for (iArr=0;iArr<DR_PAR;iArr++) 
	satellite_dat[iRow-1].params[iArr] = (double) arr_float[iArr];
    } 
    
  }
  *Nvalues = Nrows;
  return datasum;
}

unsigned long read_fits_emission_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				      struct EMISSION_DATA 
				      **emission_dat_ptr) {

  /* Input assumptions: 
     fptr is a fits file ptr, indexed to the correct HDU. 
     colltrns_dat is an COLLTRNS_DATA pointer for the data in that HDU.
     Output requirements:
     colltrns_dat should be loaded up with all the appropriate data.  
   */
  
  int Nrows, iCol, iRow, i, anynul;
  int status=0;
  float var_float;
  float arr_float[MAX_UPS];
  char *colnames[14] = {"Wavelen","Wave_Obs","Wave_Err","EM_Type",
			"Lower_Lev","Upper_Lev",
			"Scale_Z","Scale_rmJ","Scale_Upper","Scale_Lower",
			"Min_Temp","Max_Temp","Temperature","Param"};
  int colnum[14]; /* 14 names listed above */
  int Ncols = 14;
  char comment[MAXSTRLEN], version[MAXSTRLEN], filename[MAXSTRLEN];
  struct EMISSION_DATA *emission_dat;
  unsigned long datasum = 0;

  /* First, check that the is file OK and return the checksum and datasum */
  if (require_checksum) datasum = check_fits_file(fptr);

  /* Figure out how many values we have to load. */
  fits_read_key(fptr, TINT, "NAXIS2", &Nrows, comment, &status);

  if (Nrows == 0) { /* No actual data here. */
    *emission_dat_ptr = NULL;
    *Nvalues = Nrows; 
    return datasum;
  } else { /* Make space for the data */
    *emission_dat_ptr = (struct EMISSION_DATA *)
      malloc_safe(Nrows*sizeof(struct EMISSION_DATA));
    emission_dat = *emission_dat_ptr;
  }

  /* Get the column numbers */

  /* What version to read? */
  fits_read_key(fptr, TSTRING, "HDUVERS1", version, comment, &status);
  if (strncmp(version,"1.0",3) != 0) {
    if (fits_file_name(fptr, filename, &status)) {
      fits_printerror("read_fits_datafiles", status);
    }
    errmess("read_fits_datafiles","Unknown version %s in file %s",version,
	    filename);
  }
  
  for(iCol=0;iCol<Ncols;iCol++) {
    fits_get_colnum(fptr, CASEINSEN, colnames[iCol], &(colnum[iCol]), &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s column number",
	       colnames[iCol]);
    }
  }

  /* Now, read in the data. */
  for (iRow=1;iRow<Nrows+1;iRow++) {
    fits_read_col(fptr, TINT, colnum[0], iRow, 1, 1, 0,
		  &(emission_dat[iRow-1].lev_lower), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[0]);
    }

    fits_read_col(fptr, TINT, colnum[1], iRow, 1, 1, 0,
		  &(emission_dat[iRow-1].lev_upper), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[1]);
    }

    fits_read_col(fptr, TINT, colnum[2], iRow, 1, 1, 0,
		  &(emission_dat[iRow-1].type), &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[2]);
    }

    fits_read_col(fptr, TFLOAT, colnum[3], iRow, 1, 1, 0, &(var_float), 
		  &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[3]);
    }
    emission_dat[iRow-1].min_T = (double) var_float;

    fits_read_col(fptr, TFLOAT, colnum[4], iRow, 1, 1, 0, &(var_float), 
		  &anynul, &status);
    if (status!=0) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[4]);
    }
    emission_dat[iRow-1].max_T = (double) var_float;

    if (fits_read_col(fptr, TFLOAT, colnum[5], iRow, 1, MAX_UPS, 0, arr_float,
		      &anynul, &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[5]);
    } else {
      for (i=0;i<MAX_UPS;i++) emission_dat[iRow-1].T[i]=(double) arr_float[i];
    }

    if (fits_read_col(fptr, TFLOAT, colnum[6], iRow, 1, MAX_UPS, 0, arr_float,
		      &anynul, &status)) {
      fitsmess("read_fits_datafiles",status,"Error reading %s data",
	       colnames[6]);
    } else {
      for (i=0;i<MAX_UPS;i++) emission_dat[iRow-1].param[i]=
				(double) arr_float[i];
    }
  }
  *Nvalues = Nrows; 
  return datasum;
}
