#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <fitsio.h>

#include "messages.h"
#include "apec.h"

void fits_error(int status);

#define MAXDIM 100
#define ION_POP_SET_MIN 1.e-20

/* ******************************************************* */
/* read_ion_bal:
   function to read ionization balance from file such as
   AR_ionbal or AR_ionbal_test. OR ARoth_ionbal
   Eventually this will be calculated within the program.
*/
void read_ion_bal(char *ionbal_file, struct DEBUG_FILES *debug,
		  struct ION_BAL_TABLE *ionbal) {
  
  
  fitsfile *fptr;
  int hdunum, casesen;
  
  int colnum_temp, colnum_dens, colnum_Z, colnum_X;
  char column[MAXSTRLEN],comment[MAXSTRLEN];
  int loc_indx[MAXIMUM_Z+1];
  int status, hdutype, anynul;
  int Z;
  int nrows, NZ, NX, Ntemp, Ndens;
  int Tlinear, Nlinear;
  int row;
  int iTemp=0, iDens=0, iZ, iState, i;
  int nei_run;
  double sum;

  
  status = 0;

  if (fits_open_file(&fptr, ionbal_file, READONLY, &status)) {
    message("read_ion_bal","Error opening fits file %s",ionbal_file);
    fits_error(status);
  }

  hdunum = 2;
  if (fits_movabs_hdu(fptr, hdunum, &hdutype, &status)) {
    message("read_ion_bal","Error finding second HDU");
    fits_error(status);
  }
    
  casesen = FALSE;
  
  fits_get_colname(fptr, casesen, "Temperature", column,
		   &colnum_temp, &status);
  fits_get_colname(fptr, casesen, "Density", column,
		   &colnum_dens, &status);
  fits_get_colname(fptr, casesen, "Z_ELEMENT", column,
		   &colnum_Z, &status);
  fits_get_colname(fptr, casesen, "X_IONPOP", column,
		   &colnum_X, &status);

  if (status) {
    message("read_ion_bal","Error getting column names");
    fits_error(status);
  }
  
  if (fits_read_key(fptr, TINT, "NAXIS2", &nrows, comment, &status)) {
    message("read_ion_bal","Error getting number of rows.");
    fits_error(status);
  }
  if (fits_read_key(fptr, TINT, "T_NUMBER", &Ntemp, comment, &status)) {
    message("read_ion_bal","Error getting number of temperatures.");
    fits_error(status);
  }
  if (fits_read_key(fptr, TLOGICAL, "T_LINEAR", &Tlinear, comment, &status)) {
    message("read_ion_bal","Error getting temperature logical.");
    fits_error(status);
  }
  if (fits_read_key(fptr, TINT, "N_NUMBER", &Ndens, comment, &status)) {
    message("read_ion_bal","Error getting number of densities.");
    fits_error(status);
  }
  if (fits_read_key(fptr, TLOGICAL, "N_LINEAR", &Nlinear, comment, &status)) {
    message("read_ion_bal","Error getting density logical.");
    fits_error(status);
  }
  if (fits_read_key(fptr, TINT, "N_ELEMEN", &NZ, comment, &status)) {
    message("read_ion_bal","Error getting number of elements.");
    fits_error(status);
  }
  if (fits_read_key(fptr, TINT, "N_IONS", &NX, comment, &status)) {
    message("read_ion_bal","Error getting number of ions.");
    fits_error(status);
  }

  if (fits_read_key(fptr, TLOGICAL, "NEI", &nei_run, comment, &status)) {
    nei_run = FALSE;
    status = 0;
  }
  
  ionbal->NZ = NZ;
  ionbal->nei = nei_run;
  ionbal->Nrows = nrows;
  ionbal->Ntemp = Ntemp;
  ionbal->Ndens = Ndens;
  if (nei_run) {
    ionbal->T = (float *) malloc(nrows*sizeof(float));
    ionbal->dens = (float *) malloc(nrows*sizeof(float));
    ionbal->Ntemp = 0;
    ionbal->Ndens = 0;
  } else {
    ionbal->T = (float *) malloc(Ntemp*sizeof(float));
    ionbal->dens = (float *) malloc(Ndens*sizeof(float));
  }
  if ((!nei_run)&&(Ntemp*Ndens != nrows)) {
    errmess("read_ion_bal","Number of rows %d != Number of temperatures (%d) * Number of densities (%d)",nrows,Ntemp,Ndens);
  }

  /* ionbal->X = (float **) malloc(Ntemp*Ndens*sizeof(float)); */
  ionbal->X = (float **) malloc(nrows*sizeof(float)); 

  if (fits_read_col(fptr, TINT, colnum_Z, 1, 1, NZ, NULL,
		    ionbal->Z, &anynul, &status)) {
    message("read_ion_bal","Error reading element Z values.");
    fits_error(status);
  }
  /* Create the local index variable into the X vector */
  loc_indx[0] = 0;
  for (iZ=1;iZ<NZ;iZ++) {
    loc_indx[iZ] = loc_indx[iZ-1] + ionbal->Z[iZ-1] + 1;
  }

  /* Set up the real index variable to blow up if asked for an illegal Z */
  /* value, but otherwise to be correct. */

  for (iZ=0;iZ<MAXIMUM_Z+1;iZ++) ionbal->indx[iZ] = -1000;
  for (iZ=0;iZ<NZ;iZ++) ionbal->indx[ionbal->Z[iZ]] = loc_indx[iZ];

  if (nei_run) {
    for (row=0;row<nrows;row++) {
	
      if (fits_read_col(fptr, TFLOAT, colnum_temp, row+1, 1, 1, NULL,
			&(ionbal->T[row]), &anynul, &status)) {
	message("read_ion_bal","Error reading temperature values.");
	fits_error(status);
      }
	
      if (fits_read_col(fptr, TFLOAT, colnum_dens, row+1, 1, 1, NULL,
			&(ionbal->dens[row]), &anynul, &status)) {
	message("read_ion_bal","Error reading density values.");
	fits_error(status);
      }
	
      (ionbal->X)[row] = (float *) malloc(NX*sizeof(float));
      if (fits_read_col(fptr, TFLOAT, colnum_X, row+1, 1, NX, NULL,
			(ionbal->X)[row], &anynul, &status)) {
	message("read_ion_bal","Error reading ion population values.");
	fits_error(status);
      }
      
      for(i=0;i<NX;i++) {
	if ((ionbal->X)[row][i] == 0) {
	  (ionbal->X)[row][i] = ION_POP_SET_MIN;
	}
      }
    }      
  } else {
    for (iTemp=0;iTemp<Ntemp;iTemp++) {
      for (iDens=0;iDens<Ndens;iDens++) {
	row = iTemp*Ndens + iDens + 1;
	
	if (iDens==0) {
	  if (fits_read_col(fptr, TFLOAT, colnum_temp, row, 1, 1, NULL,
			    &(ionbal->T[iTemp]), &anynul, &status)) {
	    message("read_ion_bal","Error reading ion population values.");
	    fits_error(status);
	  }
	}
	
	if (iTemp==0) {
	  if (fits_read_col(fptr, TFLOAT, colnum_dens, row, 1, 1, NULL,
			    &(ionbal->dens[iDens]), &anynul, &status)) {
	    message("read_ion_bal","Error reading ion population values.");
	    fits_error(status);
	  }
	}
	
	(ionbal->X)[row-1] = (float *) malloc(NX*sizeof(float));
	if (fits_read_col(fptr, TFLOAT, colnum_X, row, 1, NX, NULL,
			  (ionbal->X)[row-1], &anynul, &status)) {
	  message("read_ion_bal","Error reading ion population values.");
	  fits_error(status);
	}
	for(i=0;i<NX;i++) {
	  if ((ionbal->X)[row-1][i] == 0) {
	    (ionbal->X)[row-1][i] = ION_POP_SET_MIN;
	  }
	}
      }      
    }
  }

  if (nei_run) {
    for (row=0; row < nrows; row++) {

      if (debug->do_ionbal && debug->ascii) 
	fprintf(debug->ionbal.ascii,"%9.3e ",(ionbal->T)[row]);

      for (iZ = 0;iZ<NZ;iZ++) {
	sum = 0.0;
	Z = ionbal->Z[iZ];

	for(iState = 0; iState < Z+1; iState++) {

	  sum += ((ionbal->X)[iTemp*Ndens + iDens])[ionbal->indx[Z]+iState];

	  if (debug->do_ionbal && debug->ascii) {
	    fprintf(debug->ionbal.ascii,"%9.3e ",
		    ((ionbal->X)[iTemp*Ndens+iDens])[ionbal->indx[Z]+iState]);
	  }
	  if (debug->do_ionbal && debug->fits) {
	    fd_write_row(&debug->ionbal.fits, 
		   (ionbal->T)[row],
		   Z, iState,
		   ((ionbal->X)[iTemp*Ndens+iDens])[ionbal->indx[Z]+iState],
		   0.0);
	  }
	}
	if (debug->do_ionbal && debug->ascii) {
	  fprintf(debug->ionbal.ascii,
		  "\nSum of pop of all ion states = %10.3f\n",sum);
	}
	if (debug->do_ionbal && debug->fits) {
	  fd_write_row(&debug->ionbal.fits, (ionbal->T)[row], Z, 0, 0.0, sum);
	}
      }
    }
  } else {
    for (iTemp=0; iTemp < ionbal->Ntemp; iTemp++) {
      for (iDens=0; iDens < ionbal->Ndens; iDens++) {

	if (debug->do_ionbal && debug->ascii) 
	  fprintf(debug->ionbal.ascii,"%9.3e ", (ionbal->T)[iTemp]);
	
	for (iZ = 0;iZ<NZ;iZ++) {
	  sum = 0.0;
	  Z = ionbal->Z[iZ];
	  
	  for(iState = 0; iState < Z+1; iState++) {
	    
	    sum += ((ionbal->X)[iTemp*Ndens + iDens])[ionbal->indx[Z]+iState];
	    
	    if (debug->do_ionbal && debug->ascii) {
	      fprintf(debug->ionbal.ascii,"%9.3e ",
		    ((ionbal->X)[iTemp*Ndens+iDens])[ionbal->indx[Z]+iState]);
	    }
	    if (debug->do_ionbal && debug->fits) {
	      fd_write_row(&debug->ionbal.fits, 
		    (ionbal->T)[iTemp],
		    Z, iState,
		    ((ionbal->X)[iTemp*Ndens+iDens])[ionbal->indx[Z]+iState],
		    0.0);
	    }
	  }
	  if (debug->do_ionbal && debug->ascii) {
	    fprintf(debug->ionbal.ascii,
		    "\nSum of pop of all ion states=%10.3f\n",sum);
	  }
	  if (debug->do_ionbal && debug->fits) {
	    fd_write_row(&debug->ionbal.fits,
			 (ionbal->T)[iTemp], Z, 0, 0.0, sum);
	}
	}
      }
    }
  }
  
  if (fits_close_file(fptr, &status)) {
    fitsmess("read_ion_bal",status,"Error closing fits file.");
  }

  if (debug->do_ionbal && debug->ascii) fflush(debug->ionbal.ascii);

}
