#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <fitsio.h>

#include "messages.h"
#include "apec.h"
#include "datatypes.h"

/* Routine reads a set of atomic data from FITS file(s).  */

char *roman_element(int Z, int J);
unsigned long read_fits_ionrec_data(int require_checksum, 
				   fitsfile *fptr, int *Nvalues, 
				   struct IONREC_DATA **); 
unsigned long read_fits_level_data(int require_checksum, 
				   fitsfile *fptr, int *Nvalues,
				   struct LEVEL_DATA **);
unsigned long read_fits_radiation_data(int require_checksum,
				       fitsfile *fptr, long *Nvalues,
				       struct LINERAD_DATA **);
unsigned long read_fits_collision_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				       struct COLLTRNS_DATA **);
unsigned long read_fits_satellite_data(int require_checksum,
				       fitsfile *fptr, int *Nvalues,
				       struct SATELLITE_DATA **);
unsigned long read_fits_emission_data(int require_checksum,
				      fitsfile *fptr, int *Nvalues,
				      struct EMISSION_DATA **);

/* Z = atomic number.  rmJ = roman number J, so 2 = II = singly ionized */

void open_file_filemap(fitsfile **fptr, int Z, int rmJ, int datatype,
		       char *ion_name, struct FILEMAP *filemap,
		       char **filename);

void read_ion_data(int require_checksum, int Z, int rmJ, 
		   struct FILEMAP *filemap, int *maxNlev,
		   struct ELEMENT_ION_STATE *state) {

  fitsfile *fptr;
  char *filename;
  char ion[16];
  int status=0; 

  /* state points to a bunch of set-up vectors. Time to load up the  */
  /* ion_inf variable, which is the only non-array variable in the bunch. */
  /* The others will be handled as they are read in. */

  
  /* Find filename, open file, move to appropriate HDU. */
  sprintf(ion, "%s", roman_element(Z,rmJ-1));
  open_file_filemap(&fptr, Z, rmJ, DATATYPE_IONREC, ion, 
		    filemap, &filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_ionrec=
      read_fits_ionrec_data(require_checksum, fptr, 
			    &(state->ion_inf->nionrec),
			    &(state->ionrec_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
  } else { /* No data */
    state->ion_inf->nionrec = 0;
    state->ion_inf->cs_ionrec = 0;
  }

  open_file_filemap(&fptr, Z, rmJ, DATATYPE_LEVEL, ion, filemap, &filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_elvls=read_fits_level_data(require_checksum, fptr,
						  &(state->ion_inf->nlev),
						  &(state->lev_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
    if (state->ion_inf->nlev > *maxNlev) *maxNlev = state->ion_inf->nlev;
  } else {
    state->ion_inf->nlev = 0;
    state->ion_inf->cs_elvls = 0;
  }

  open_file_filemap(&fptr, Z, rmJ,DATATYPE_RADIATION, ion, filemap, 
		    &filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_lines=
      read_fits_radiation_data(require_checksum, fptr, 
			       &(state->ion_inf->nwv),
			       &(state->linerad_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
  } else {
    state->ion_inf->nwv = 0;
    state->ion_inf->cs_lines = 0;
  }

  open_file_filemap(&fptr,Z,rmJ,DATATYPE_E_COLLISION,ion,filemap,&filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_ecoll = 
      read_fits_collision_data(require_checksum, fptr, 
			       &(state->ion_inf->nexc),
			       &(state->colltrns_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
  } else {
    state->ion_inf->nexc = 0;
    state->ion_inf->cs_ecoll = 0;
  }

  open_file_filemap(&fptr,Z,rmJ,DATATYPE_P_COLLISION,ion,filemap,&filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_pcoll=
      read_fits_collision_data(require_checksum, fptr,
			       &(state->ion_inf->npro),
			       &(state->protrns_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
  } else {
    state->ion_inf->npro = 0;
    state->ion_inf->cs_pcoll = 0;
  }

  open_file_filemap(&fptr,Z,rmJ,DATATYPE_SATELLITE,ion,filemap,&filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_drsat = 
      read_fits_satellite_data(require_checksum, fptr,
			       &(state->ion_inf->nsatel), 
			       &(state->satellite_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
  } else {
    state->ion_inf->nsatel = 0;
    state->ion_inf->cs_drsat = 0;
  }

  open_file_filemap(&fptr,Z,rmJ,DATATYPE_EMISSION,ion,filemap,&filename);
  if (fptr!=NULL) {
    state->ion_inf->cs_emiss = 
      read_fits_emission_data(require_checksum, fptr,
			       &(state->ion_inf->nemiss), 
			      &(state->emission_dat));
    if (fits_close_file(fptr, &status)) {
      fitsmess("read_ion_data",status,"Error closing file.");
    }
  } else {
    state->ion_inf->nemiss = 0;
    state->ion_inf->cs_emiss = 0;
  }


}
