FUNCTION calc_p_upsilon, Z, rmJ, ll, lu, T, SILENT=silent, FILE=file
;
;  Z is the element, rmJ the roman number of the ion, ll the lower
;  level, lu the upper level, and T the temperature, which may be a
;  vector.
;

multi = 0   ; single transition only
KB = 8.617e-8
E_lu = calc_energy(Z,rmJ,lu)
E_ll = calc_energy(Z,rmJ,ll)
if ((E_lu lt 0.0)or(E_ll lt 0.0)) then begin
    if (n_elements(silent) eq 0) then print,'No appropriate energy data found'
    return,0*T
endif
dE = E_lu - E_ll
chi = dE/(KB*T*1.e3)
N_T = n_elements(T)
;
; Load collision strength structures
;
read_Pcoll, Z, rmJ, data, header, found=found, FILE=file
if (found ne 1) then begin
    if (n_elements(silent) eq 0) then print,'No appropriate data found'
    return,0*T
endif
;
; Now, does the appropriate ll, lu exist in this data structure?
;
Indx = where((data.lower_lev eq ll)and(data.upper_lev eq lu),count)
if (count eq 0) then begin
    if (n_elements(silent) eq 0) then $
      print,strcompress('No such transition '+string(ll)+' ->'+string(lu)+$
                        ' in the data structure')
    return,0*T
endif
if (count gt 1) then begin
;
;  This could be because the two transitions rates should be summed,
;  or because the transition type is interpolation and the multiple
;  transitions handle different temperature values.  None the less,
;  we'll print a warning at least.
;
    if (n_elements(silent) eq 0) then $
      print,strcompress('More than one such transition '+string(ll)+' ->'+$
                        string(lu)+' in the data structure.')
    multi = 1  ; multiple transitions.
endif

Ntr = n_elements(Indx)
result = fltarr(Ntr,N_T)

for iTr=0,Ntr-1 do begin
;
; OK, at this point, we've got a single good transition
;
    tr = data(Indx(iTr))
;
; Now, check the transition temperatures
;
    gp = where((T ge tr.min_temp) and (T le tr.max_temp),count_gp)
    bp = where((T lt tr.min_temp) and (T gt tr.max_temp),count_bp)
;
; This is a clever section designed to fix all the interpolation
; issues--ie, is the max or min temperature included in the interpolation?
; What we do is check to see which case we're in, and then fix gp and
; bp appropriately and set the coeff_type to the appropriate normal case.
;
    fix_ups_interp, tr, T, gp, bp, count_gp, count_bp

    if (count_gp eq 0) then begin
;        if (n_elements(silent) eq 0) then $
;          print,'No temperatures with allowed range for transition.'
        result(iTr,*) = 0.0
    endif else begin

        if (count_bp gt 0) then result(iTr,bp) = 0.0 ; out-of-range pts = zero
    
        if ((tr.Coeff_Type ge 100)and (tr.Coeff_Type le 120)) then begin
            AP = indgen(tr.Coeff_Type-100)
            result(iTr,gp) = 10^interpol(alog10(tr.Coeff_OM(AP)),$
                                         alog10(tr.Temperature(AP)),$
                                         alog10(T(gp)))
        endif

        if ((tr.Coeff_Type ge 200)and (tr.Coeff_Type le 220)) then begin
            AP = indgen(tr.Coeff_Type-200)
            result(iTr,gp) = 10^interpol(alog10(tr.Coeff_OM(AP)),$
                                         alog10(tr.Temperature(AP)),$
                                         alog10(T(gp)))
        endif

        if ((tr.Coeff_Type ge 300)and (tr.Coeff_Type le 320)) then begin 
            AP = indgen(tr.Coeff_Type-300)
            result(iTr,gp) = 10^interpol(alog10(tr.Coeff_OM(AP)),$
                                         alog10(tr.Temperature(AP)),$
                                         alog10(T(gp)))
            result(iTr,gp) = (result(iTr,gp)/8.63e-6)*exp(chi(gp))*$
              (sqrt(T(gp))*calc_statwt(Z,rmJ,ll))
        endif
        
        if ((tr.Coeff_Type ge 400)and (tr.Coeff_Type le 420)) then begin 
            AP = indgen(tr.Coeff_Type-400)
            result(iTr,gp) = 10^interpol(alog10(tr.Coeff_OM(AP)),$
                                         alog10(tr.Temperature(AP)),$
                                         alog10(T(gp)))
            result(iTr,gp) = (result(iTr,gp)/8.63e-6)*exp(chi(gp))*$
              (sqrt(T(gp))*calc_statwt(Z,rmJ,ll))
        endif
        
        if ((tr.Coeff_Type gt 10)and(tr.Coeff_Type lt 15)) then $ ; Chianti
          result(iTr,gp) = calc_chianti(tr, dE, T(gp))    
        
        if ((tr.Coeff_Type gt 30)and(tr.Coeff_Type lt 34)) then $ ; SGC
          result(iTr,gp) = calc_sgc(tr, Z, T(gp))
        
        if ((tr.Coeff_Type eq 41)or(tr.Coeff_Type eq 42)) then begin ; Kato
            result(iTr,gp) = calc_kato(tr, T(gp))
        endif
        
        if (tr.Coeff_Type eq 1) then begin ; Burgess&Tully
            print,'Not implemented yet.'
            stop
        endif
    endelse
endfor

final = fltarr(N_T)
for iT=0,N_T-1 do final(iT) = total(result(*,iT))

return, final
end
