FUNCTION calc_levpop, Z, rmJ, lvls, T, dene, SILENT=silent,COLLFILE=collfile,$
                      LINEFILE=linefile, LEVFILE=levfile
;
; Calculate level population of the ion {Z, rmJ}, using only the 
; levels in the array lvls. Temperature T or electron density dene may
; be a vector, but not both.
; If lvls = -1, use the entire ion.
;
if (n_params() lt 5) then begin
    print,'calc_levpop calculates the level population of an ion, using'
    print,'some or all of its energy levels. Call it as: '
    print,'pop = calc_levpop(Z, rmJ, levels, T, dene)'
    print,'where Z, rmJ is the element Z and ion state rmJ, 1 = neutral'
    print,'levels is an array with the levels to consider, or -1 to do all.'
    print,'T is the temperature (in Kelvin).'
    print,'dene is the electron density in cm^-3.'
    print,'Either T or dene may be an array, but not both.'
    print,'The result, pop, is a vector (if T and dene are scalars) or an' 
    print,'array of the form (NTemp, Nlevels) or (Ndensities, Nlevels).'
    return,0.0
endif

if (n_elements(levfile) ne 0) then begin
    print,'FILE command not yet implemented in calc_einstein_A'
    print,'Bombing out.'
    stop
endif
if ((n_elements(T) gt 1)and(n_elements(dene) gt 1)) then begin
    if (n_elements(silent)) then print,$
        'Cannot have temperature and density both be arrays.'
    return, 0.0
endif
Nmat = 1
if (n_elements(T) gt 1) then begin
    Tvec = 1
    Nvec = 0
    Nmat = n_elements(T)
endif

if (n_elements(dene) gt 1) then begin
    Nvec = 1
    Tvec = 0
    Nmat = n_elements(dene)
endif

read_elev, Z, rmJ, data, header,/silent, FILE=levfile

if (n_elements(lvls) eq 1) then begin
    if (lvls(0) eq -1) then begin 
        nlev = n_elements(data)
        lvls = indgen(nlev) + 1
    endif  else begin
        if (n_elements(silent) ne 0) then print,'Only one level to solve for!'
        return, 0.0
    endelse
endif else begin
    nlev = n_elements(lvls)
endelse

lsrt = sort(data(lvls).energy)
lvls = lvls(lsrt)

A_mat = dblarr(Nmat, nlev, nlev)
A_mat(*,nlev-1,0) = 1.0

for iMat = 0,Nmat-1 do begin
    for ll=1,nlev-1 do begin
        A_mat(iMat,ll-1,0) = 1.0     ; A_max(column, row)
        for ul=ll+1,nlev do begin  
            exc   = calc_rate(Z,rmJ,lvls(ll-1),lvls(ul-1),T,/silent,$
                              FILE=collfile)
            deexc = calc_derate(Z,rmJ,lvls(ll-1),lvls(ul-1),T,/silent,$
                                FILE=collfile)
            rad   = total(calc_einstein_A(Z,rmJ,lvls(ll-1),lvls(ul-1),/silent))
            A_mat(*,ll-1,ul-1) = dene*exc
            if (ll ne 1) then A_mat(*,ul-1,ll-1) = dene*deexc + rad
;        print, lvls(ll-1), lvls(ul-1), dene*exc, dene*deexc, rad
        endfor
    endfor

    for diag=2,nlev do begin
        Diag_El = total(A_mat(iMat,*,diag-1)) 
        for ll = 1, diag-1 do begin
            Diag_El = Diag_El + $
              total(calc_einstein_A(Z,rmJ,lvls(ll-1),lvls(diag-1),/silent))
        endfor
        A_mat(iMat,diag-1,diag-1) = -Diag_El
    endfor
endfor
;
; Matrix and vectors are all set up.  Now, solve for the level population.
;
; print,A_mat

levpop = dblarr(Nmat, nlev)

for iMat=0,Nmat-1 do begin
    B_vec = dblarr(nlev)
    B_vec(0) = 1.0
    Matrix = reform(A_mat(iMat,*,*))
    print,Matrix
    ludc, Matrix, index, /double
    levpop(iMat,*) = LUSOL(Matrix, index, B_vec,/double)
endfor
if (Nmat eq 1) then levpop = reform(levpop)

return, levpop

end
