PRO write_Eline, filename, Z, rmJ, data, COMMENTS=comments, WV_REF=wvref,$
                 WV_OBS_REF=wvobsref, EINA_REF=einaref, NANVALUE=nanvalue
;
;  Writes a Eline FITS file using the data in the structure data, for
;  element Z and ion stage rmJ.  The code first checks to see if
;  filename exists.  If it doesn't, it creates filename and then
;  writes the primary header.  If it does, the code checks to make
;  sure this particular ion stage hasn't already been written to the
;  file.  Assuming all is OK, we are positioned at the end of the file.
;  Then the header is created, and the data is written to the file.
;

;
;  Check to see if file exists
;
result = findfile(filename,count=count)
if (count eq 0) then begin      ; must create file and primary HDU
    fxhmake, primary_header, /extend, /initialize, /date
    fxaddpar, primary_header, 'EM_LINES','Emission Line Data
    fxaddpar, primary_header, 'FILENAME','IDL_routine','Source of data'
    fxaddpar, primary_header, 'ORIGIN','CXC','Chandra X-ray Center'
    fxaddpar, primary_header, 'HDUCLASS','ATOMIC','Atomic data'
    fxaddpar, primary_header, 'HDUCLAS1','EM_LINES','Emission Line data'
    version = '1.1.0'
    fxaddpar, primary_header, 'HDUVERS1', version, 'Version of datafile'
    fxwrite, filename, primary_header
endif

if (count gt 1) then begin
    print,'More than 1 file exists with this name: ',result
    stop
endif

if (count eq 1) then begin
    Ext = match_header(filename, Z, rmJ, 'EM_LINES')
    if (Ext gt 0) then begin
        print,'Extension ',Ext,' already has data for Z, rmJ = ',Z,rmJ,$
          ' in file ',filename
        return
    endif
endif
;
;  Now we're all set, ready to create the new header.
;
integer = 1L
float = 1.0
array = fltarr(10)
strings = '01234567890123456789'
nrows = n_elements(data)
extname = ionname(Z,rmJ)
fxbhmake, header, nrows, extname, 'Ion Name'
fxaddpar, header, 'HDUCLASS', 'ATOMIC', 'Atomic Data'
fxaddpar, header, 'HDUCLAS1','EM_LINES', 'Emission line tables'
fxaddpar, header, 'ELEMENT', Z, 'Number of protons in element'
fxaddpar, header, 'ION_STAT',rmJ-1,'Ion state (0 = neutral)'
fxaddpar, header, 'ION_NAME',extname, 'Ion Name'
fxaddpar, header, 'N_LINES',nrows,'Number of emission lines'
version = '1.1.0'
fxaddpar, header, 'HDUVERS1', version, 'Version of datafile'
fxbaddcol, 1, header, integer, 'Upper_Lev'
fxbaddcol, 2, header, integer, 'Lower_Lev'
fxbaddcol, 3, header, float,   'Wavelen ',TUNIT='Angstrom'
fxbaddcol, 4, header, float,   'Wave_Obs',TUNIT='Angstrom'
fxbaddcol, 5, header, float,   'Wave_Err',TUNIT='Angstrom'
fxbaddcol, 6, header, float,   'Einstein_A',TUNIT='s**-1'
fxbaddcol, 7, header, float,   'Ein_A_Err',TUNIT='s**-1'
fxbaddcol, 8, header, strings, 'Wave_Ref'
fxbaddcol, 9, header, strings, 'Wv_Obs_Ref'
fxbaddcol,10, header, strings, 'Ein_A_Ref'
;
; Now add any additional comments
;
for i=0,n_elements(comments)-1 do begin
    if (strpos(comments(i),'COMMENT') eq 0) then begin
        commstr = strmid(comments(i),8)
        add_reference, header, commstr, /header
    endif else begin
        add_reference, header, comments(i), /header
    endelse
endfor
;
;  Now open the file and write the new header
;
fxbcreate, unit, filename, header
;
;  Write the data
;
for i=0L,nrows-1 do begin
    fxbwrite, unit, data(i).upper_lev, 1, i+1
    fxbwrite, unit, data(i).lower_lev, 2, i+1
    fxbwrite, unit, data(i).wavelen,   3, i+1, NANVALUE=nanvalue
    fxbwrite, unit, data(i).wave_obs,  4, i+1, NANVALUE=nanvalue
    fxbwrite, unit, data(i).wave_err,  5, i+1, NANVALUE=nanvalue
    fxbwrite, unit, data(i).Einstein_A,6, i+1, NANVALUE=nanvalue
    fxbwrite, unit, data(i).Ein_A_Err, 7, i+1, NANVALUE=nanvalue
    if (n_elements(wvref) eq 0) then begin
        fxbwrite, unit, data(i).Wave_Ref,8, i+1
    endif else begin
        fxbwrite, unit, wvref         ,8, i+1
    endelse
    if (n_elements(wvobsref) eq 0) then begin
        fxbwrite, unit, data(i).Wv_Obs_Ref,9, i+1
    endif else begin
        fxbwrite, unit, wvobsref      ,9, i+1
    endelse
    if (n_elements(einaref) eq 0) then begin
        fxbwrite, unit, data(i).Ein_A_Ref,10, i+1
    endif else begin
        fxbwrite, unit, einaref,10, i+1
    endelse
endfor
;
; Close the file
;
fxbfinish, unit

return
end
