%
% Routines handling inclusion/exclusion of elements in the model
% calculation and also provide the possibility of different 
% abundances for different elements.
%
% Ronnie Hoogerwerf, SAO, 04/10/2002.
%
%

variable elems_min_C = ["abundN", "abundO", "abundF", "abundNe", "abundMg", "abundAl", "abundSi", "abundS", "abundAr", "abundCa", "abundFe", "abundNi"];

define AM_Elements_show () {

   % Routine shows which elements are included in the model and
   % what their respective abundances are.
   
   variable i;

   
   message("\n\n");
   message("Element ( Z)   Abundance");
   message("========================");
   message(" H      ( 1)    1.000000");
   message(" He     ( 2)    1.000000");
   for (i = 0; i < length(AM->Elements); i++) {
      vmessage("%2s      (%2d)    %f",
	       AM->Elements[i], AM->ElementsZ[i],
	       AM->Abundances[i]);
   }
   message("\n\n");

}

define AM_include(elem) {

   % Include an element so that its lines and continuum are include
   % in the model spectrum

   variable abund, mdls;

   if (Integer_Type == typeof(elem)) {
      abund = "abund" + AM->Elements[where(AM->ElementsZ == elem)][0];
   } else if (String_Type == typeof(elem)) {
      abund = "abund" + elem;
   } else {
      verror("Received incorrect input for AM_include. Enter\nElement name or Atomic number");
   }

   mdls = AM_get_apec_modelnames_only();

   if (length(mdls) > 0) {
      variable k, sherpa_cmnd;
      foreach (mdls) {
	 
	 k = ();
	 
	 sherpa_cmnd = "CREATE " + k + "." + abund + ".VALUE = 1.0";
	 AM_sherpa_eval(sherpa_cmnd);
	 sherpa_cmnd = "thaw " + k + "." + abund;
	 AM_sherpa_eval(sherpa_cmnd);
      }
   }

}

define AM_exclude(elem) {

   % Exinclude an element so that its lines and continuum are not 
   % excluded in the model spectrum

   variable abund, tmp;

   if (Integer_Type == typeof(elem)) {
      abund = "abund" + AM->Elements[where(AM->ElementsZ == elem)][0];
   } else if (String_Type == typeof(elem)) {
      abund = "abund" + elem;
   } else {
      verror("Received incorrect input for AM_deinclude. Enter\nElement name or Atomic number");
   }
   
   variable mdls = AM_get_apec_modelnames_only();

   if (length(mdls) > 0) {
      variable k, sherpa_cmnd;
      foreach (mdls) {
	 
	 k = ();
	 
	 sherpa_cmnd = "CREATE " + k + "." + abund + ".VALUE = 0.0";
	 AM_sherpa_eval(sherpa_cmnd);
	 sherpa_cmnd = "freeze " + k + "." + abund;
	 AM_sherpa_eval(sherpa_cmnd);
      }
   }

}

define AM_include_all() {

   % Include all metalic elements so that their lines and continuum 
   % are include in the model spectrum

   variable abund, elem;
   variable mdls = AM_get_apec_modelnames_only();
   

   foreach (AM->Elements) {
      elem = ();
      abund = "abund" + elem;
      if (length(mdls) > 0) {
	 variable k, sherpa_cmnd;
	 foreach (mdls) {
	    
	    k = ();
	    
	    sherpa_cmnd = "CREATE " + k + "." + abund + ".VALUE = 1.0";
	    AM_sherpa_eval(sherpa_cmnd);
	    sherpa_cmnd = "thaw " + k + "." + abund;
	    AM_sherpa_eval(sherpa_cmnd);
	 }
      }

   }
}

define AM_exclude_all() {

   % Exclude all metalic elements so that their lines and continuum 
   % are not included in the model spectrum

   variable abund, elem;
   variable mdls = AM_get_apec_modelnames_only();

   foreach (AM->Elements) {
      elem = ();
      abund = "abund" + elem;

      if (length(mdls) > 0) {
	 variable k, sherpa_cmnd;
	 foreach (mdls) {
	    
	    k = ();
	    
	    sherpa_cmnd = "CREATE " + k + "." + abund + ".VALUE = 0.0";
	    AM_sherpa_eval(sherpa_cmnd);
	    sherpa_cmnd = "freeze " + k + "." + abund;
	    AM_sherpa_eval(sherpa_cmnd);
	 }
      }

   }
}

define AM_setabund() { 

   %
   % specify the abundance of one or all metals
   %
   variable abund;
   variable elem;

   if ((_NARGS != 1) and (_NARGS != 2)) {
      message("Call as:");
      message("AM_setabund(abund [, elem]) where");
      message("abund = abundance (in solar units)");
      message("elem = element name or atomic number");
      _pop_n(_NARGS);
      return;
   }

   if (_NARGS == 1) {
      (abund) = ();
      AM->Abundances[where(AM->ElementsZ > 2)] = abund;
   } else {
      (abund, elem) = ();
      if (Integer_Type == typeof(elem)) {
	 AM->Abundances[where(AM->ElementsZ == elem)] = abund;
      } else {
	 AM->Abundances[where(AM->Elements == elem)] = abund;
      }	 
   }
}

define AM_linkabund() { 

   %
   % specify if abundances are all the same or 
   % abundances can all be different 
   %
   variable abund = NULL, model = NULL;

   
   if ((_NARGS != 1) and (_NARGS != 2)) {
      message("Call as:");
      message("AM_linkabund(model [, abund]) where");
      message("model = name of the sherpa model");
      message("abund = set abundance of C to abund");
      _pop_n(_NARGS);
      return;
   }

   if (_NARGS == 1) {
      model = ();
   } else {
      (model, abund) = ();
   }

   variable sherpa_cmnd, elem, status;

   if (NULL != abund) {
      sherpa_cmnd = model + ".abundC = " + string(abund);
      status = sherpa_eval(sherpa_cmnd);
      if (status == -1) {
	 vmessage("\nAM_linkabund: Modelname \"%s\" does not exist, or is not an apec model.", model);
	 return;
      }
   }      
   
   foreach (elems_min_C) {
      elem = ();
      %
      sherpa_cmnd = model + "." + elem + " => " + model + ".abundC";
      status = sherpa_eval(sherpa_cmnd);
      if (status == -1) {
	 vmessage("\nAM_linkabund: Modelname \"%s\" does not exist, or is not an apec model.", model);
	 return;
      }
   }

}


define AM_unlinkabund() { 

   %
   % specify if abundances are all the same (link = 1) or 
   % abundances can all be different (link = 0)
   %
   variable model = NULL;

   if (_NARGS != 1) {
      message("Call as:");
      message("AM_unlinkabund(model) where");
      message("model = name of the sherpa model");
      _pop_n(_NARGS);
      return;
   }

   model = ();

   variable sherpa_cmnd, elem, status;

   foreach (elems_min_C) {
      elem = ();
      %
      sherpa_cmnd = "unlink " + model + "." + elem;
      status = sherpa_eval(sherpa_cmnd);
      if (status == -1) {
	 vmessage("\nAM_unlinkabund: Modelname \"%s\" does not exist, or is not an apec model.", model);
	 return;
      }
   }
}



