%
% SLang implementation of some of the routines defined in the 
% Numerical Recipes (Second Edition).
%
% R. Hoogerwerf, SAO, 04/25/2002
%

uses("MATH-functions","SL-utilities");
use_namespace("Math");


%-----

define is_number(x) {

   %
   % Check if x is a number
   %

   variable type = typeof(x);

   if (type == Array_Type) type = _typeof(x);

   if (orelse 
       {type == Short_Type}
       {type == UShort_Type}
       {type == Integer_Type}
       {type == UInteger_Type}
       {type == Long_Type}
       {type == ULong_Type}
       {type == Float_Type}
       {type == Double_Type}
       {type == Complex_Type}) {
       return 1;
    } else {
       return 0;
    }
}

%-----

define Gauss (x,mu,sigma) {

   variable gauss;

   if (sigma == 0.0) {
      error("Gauss: sigma is equal to 0");
   }

   gauss = (1.0/sqrt(2.0 * Math->pi * sigma^2)) * 
       exp(-0.5 * (x - mu)^2 / sigma^2);
 
   return gauss;

}

%-----

define Int_Gauss (x1,x2,mu,sigma) {

   variable igauss;
   variable sqrt2 = sqrt(2.0);
   variable y1, y2;

   if (sigma == 0.0) {
      error("Int_Gauss: sigma is equal to 0");
   }

   y1 = (x1 -mu) / (sqrt2 * sigma);
   y2 = (x2 -mu) / (sqrt2 * sigma);

   igauss = 0.5 * (Math->erf(y2) - Math->erf(y1));

   return igauss;

}

%-----

define Lorentz(x,mu,fwhm) {

   variable lorentz;
   variable fwhm2 = fwhm / 2.0;

   if (fwhm == 0.0) {
      error("Lorentz: fwhm is equal to 0");
   }

   lorentz = (1.0/pi) * (fwhm2 / ((x-mu)^2 + fwhm2^2));
   
   return lorentz;
       
}

%-----

define Int_Lorentz(x1,x2,mu,fwhm) {

   variable ilorentz;
   variable fwhm2 = fwhm / 2.0;
   variable y1, y2;

   if (fwhm == 0.0) {
      error("Int_Lorentz: fwhm is equal to 0");
   }
   
   y1 = (x1 - mu) / fwhm2;
   y2 = (x2 - mu) / fwhm2;

   ilorentz = (atan(y2) - atan(y1)) / Math->pi;

   return ilorentz;

}

%-----
%-----



%-----

define int(x) {
   
   %
   % Returns the integer part of x
   %

   return Global->int(x);

}

%-----

define nint(x) {

   %
   % Returns the nearest integer to x
   %

   if (Array_Type == typeof(x)) {
      x[where(x >= 0.0)] = Global->int(x[where(x>=0.0)] + 0.5);
      x[where(x < 0.0)]  = Global->int(x[where(x< 0.0)] - 0.5);
      return x;
   } else {
      if (x >= 0.) {
	 return Global->int(x+0.5);
      } else {
	 return Global->int(x-0.5);
      }
   }

}

%-----

define floor (x) {

   %
   % Mimics Lisp function (floor x)
   %

   variable floor1, floor2;

   floor1 = Math->int_left(x);
   floor2 = x - floor1;

   return floor1, floor2;

}

%-----

define ceiling (x) {

   %
   % Mimics Lisp function (ceiling x)
   %

   variable ceiling1, ceiling2;

   ceiling1 = Math->int_right(x);
   ceiling2 = x - ceiling1;

   return ceiling1, ceiling2;

}

%-----

define int_left(x) {

   variable i = Math->int(x);

   if (Array_Type == typeof(x)) {
      i[where((i != x) and (i < 0))] = i[where((i != x) and (i < 0))] - 1;
      return i;
   } else if (i < 0) {
      return i - 1;
   } else {
      return i;
   }

}

%-----

define int_right(x) {

   variable i = Math->int(x);

   if (Array_Type == typeof(x)) {
      i[where((i != x) and (i > 0))] = i[where((i != x) and (i > 0))] + 1;
      return i;
   } else if (i > 0) {
      return i + 1;
   } else {
      return i;
   }
}

%-----

define is_odd(i) {
   
   if (Integer_Type != typeof(i)) {
      verror("Math->is_odd expects argument of Integer_Type; received %s.",
	     Utils->print_typeof(i));
   }

   if (0 == i mod 2) {
      return 0;
   } else {
      return 1;
   }
}

%-----

define is_even(i) {
   
   if (Integer_Type != typeof(i)) {
      verror("Math->is_odd expects argument of Integer_Type; received %s.",
	     Utils->print_typeof(i));
   }

   if (0 == i mod 2) {
      return 1;
   } else {
      return 0;
   }
}

%-----
%-----
%-----
%-----
%-----
%-----
%-----
%-----
%-----
%-----
%-----
%-----


