/** \file ApedDatabase.h
    \brief Declaration of ApedDatabase class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_ApedDatabase_h
#define profit_ApedDatabase_h

#include "AtomDbImp.h"
#include "profit/Table.h"

#include <string>

namespace profit {

  /** \class ApedDatabase
      \brief Class encapsulating access to the atomic database.
  */
  class ApedDatabase : public AtomDbImp {
    public:
      /** \brief Create an atomic database accessor object.
          \param file_name The name of the atomic database file.
      */
      ApedDatabase(const std::string & file_name);

      /** \brief Discard database entries (in memory) whose energy falls outside an energy range.
          \param min_energy Energy object with minimum value of range.
          \param max_energy Energy object with maximum value of range.
      */
      virtual void filter(const IEnergy & min_energy, const IEnergy & max_energy);

      /** \brief Discard database entries (in memory) whose emissivity is less than the given threshold.
          \param min_emissivity Minimum emissivity accepted.
      */
      virtual void filter(double min_emissivity);

      /// \brief Get container of atomic lines which match the current criteria for line selection.
      virtual const AtomicLineCont & getMatches() const;

    private:
      InputTable m_peak_emis_table;
      mutable AtomicLineCont m_matches;
  };

}

#endif
