/** \file DataEntryWidget.cxx
    \brief Implementation of DataEntryWidget class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <algorithm>
#include <stdexcept>

#include "TGFrame.h"
#include "TGLabel.h"
#include "TGLayout.h"
#include "TGTextEntry.h"

#include "profit/DataEntryWidget.h"

ClassImp(profit::DataEntryWidget)

namespace profit {

  DataEntryWidget::DataEntryWidget(const std::string & style, const std::string & label, const std::string & content):
    m_style(style), m_label_text(label), m_content(content), m_centerY_expandX(kLHintsExpandX | kLHintsCenterY),
    m_centerY_pad_all(kLHintsCenterY, 5, 5, 5, 5), m_parent(0), m_frame(0), m_label(0), m_widget(0), m_button(0),
    m_text_entry(0), m_tied_widget(0), m_activated(true) {
  }

  DataEntryWidget::~DataEntryWidget() {
    clearDisplay();
  }

  void DataEntryWidget::buildGui(TGCompositeFrame * parent, unsigned int width, unsigned int height) {
    m_parent = parent;
    m_frame = new TGHorizontalFrame(parent, 10, 10);
    if (0 == m_frame) {
      clearDisplay();
      throw std::runtime_error("DataEntryWidget constructor failed to allocate memory for TGHorizontalFrame");
    }
    parent->AddFrame(m_frame, &m_centerY_expandX);

    if (m_style == "check button") {
      m_button = new TGCheckButton(m_frame, m_label_text.c_str());
      m_button->Connect("Clicked()", "profit::DataEntryWidget", this, "handleClicked()");

      m_widget = m_button;
      m_frame->AddFrame(m_widget, &m_centerY_pad_all);
    } else if (m_style == "text entry") {
      m_label = new TGLabel(m_frame, m_label_text.c_str());
      if (0 == m_label) {
        clearDisplay();
        throw std::runtime_error("DataEntryWidget constructor failed to allocate memory for TGLabel");
      }
      m_frame->AddFrame(m_label, &m_centerY_pad_all);

      m_text_entry = new TGTextEntry(m_frame, "");
      m_text_entry->Connect("TextChanged(char *)", "profit::DataEntryWidget", this, "handleTextEntry(const char *)");

      m_widget = m_text_entry;
      m_frame->AddFrame(m_widget, &m_centerY_expandX);
    }
    if (0 == m_widget) {
      clearDisplay();
      throw std::runtime_error("DataEntryWidget constructor failed to allocate memory for " + m_style +
        (m_style.empty() ? "" : " ") + "widget");
    }
    set(m_content);
    activate(m_activated);
  }

  void DataEntryWidget::handleClicked() {
    bool is_on = (kTRUE == m_button->IsOn());
    if (0 != m_button) m_content = is_on ? "on" : "off";
    if (0 != m_tied_widget) m_tied_widget->set(is_on ? "off" : "on");
    for (std::list<DataEntryWidget *>::iterator itor = m_controlled.begin(); itor != m_controlled.end(); ++itor) {
      (*itor)->activate((m_content != "off" && m_content != "0") ? true: false);
    }
  }

  void DataEntryWidget::handleTextEntry(const char * content) {
    m_content = content;
  }

  void DataEntryWidget::clearDisplay() {
    if (0 != m_frame) {
      if (0 != m_widget) m_frame->RemoveFrame(m_widget);
      if (0 != m_label) m_frame->RemoveFrame(m_label);
      if (0 != m_parent) m_parent->RemoveFrame(m_frame);
    }
    delete m_widget;
    delete m_label;
    delete m_frame;
    m_text_entry = 0;
    m_widget = 0;
    m_label = 0;
    m_frame = 0;
    m_parent = 0;
  }

  void DataEntryWidget::tie(DataEntryWidget * widget) { m_tied_widget = widget; }

  void DataEntryWidget::control(DataEntryWidget * widget) {
    std::list<DataEntryWidget *>::iterator itor = std::find(m_controlled.begin(), m_controlled.end(), widget);
    if (m_controlled.end() == itor) {
      m_controlled.push_back(widget);
    }
  }

  void DataEntryWidget::activate(bool is_on) {
    if (0 != m_button) {
      if (is_on) set(m_content);
      else m_button->SetState(kButtonDisabled);
    } else if (0 != m_text_entry) {
      m_text_entry->SetEnabled(is_on ? kTRUE : kFALSE);
    }
    m_activated = is_on;
  }

  DataEntryWidget::DataEntryWidget(): m_label_text(), m_content(), m_parent(0), m_frame(0), m_label(0), m_widget(0),
    m_button(0), m_text_entry(0), m_tied_widget(0), m_activated(true) {
  }

}
