/** \file Spectrum.cxx
    \brief Implementation of Spectrum class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/SpecFile.h"
#include "profit/Spectrum.h"
#include "profit/Table.h"

#include <map>
#include <stdexcept>
#include <utility>

namespace profit {

  Spectrum::Spectrum(const SpecFile & spec_file, int spec_index): m_spec_file(&spec_file),
    m_e_min(spec_file.getInputTable()->getNumRecords()), m_e_max(spec_file.getInputTable()->getNumRecords()),
    m_counts(spec_file.getInputTable()->getNumRecords()), m_spec_index(spec_index) {
    load(*spec_file.getInputTable());
  }

  Spectrum::Spectrum(const SpecFile & spec_file, const InputTable & ebounds_table): m_spec_file(&spec_file),
    m_e_min(ebounds_table.getNumRecords()), m_e_max(ebounds_table.getNumRecords()),
    m_counts(spec_file.getInputTable()->getNumRecords()), m_spec_index(-1) {
    load(*spec_file.getInputTable(), ebounds_table);
  }

  Spectrum::Spectrum(const SpecFile & spec_file, const InputTable & ebounds_table, int spec_index): m_spec_file(&spec_file), 
    m_e_min(ebounds_table.getNumRecords()), m_e_max(ebounds_table.getNumRecords()), m_counts(), m_spec_index(spec_index) {
    load(*spec_file.getInputTable(), ebounds_table);
  }

  void Spectrum::select(double min_energy, double max_energy) {
    std::vector<double>::size_type ii = 0;
    // Iterate over minimum energies, skipping energies < min_energy.
    for (ii = 0; ii < m_e_min.size() && m_e_min[ii] < min_energy; ++ii) {}
    // ii holds the position of the first bin whose minimum energy is >= min_energy.
    std::vector<double>::size_type low_cut = ii;

    // Find last energy bin whose maximum is <= the maximum value.
    for (ii = low_cut; ii < m_e_max.size() && m_e_max[ii] <= max_energy; ++ii) {}
    // ii holds the position of one bin past the last bin whose maximum energy is <= max_energy.
    std::vector<double>::size_type high_cut = ii;

    // Apply cuts to all three arrays.
    m_e_min = std::vector<double>(m_e_min.begin() + low_cut, m_e_min.begin() + high_cut);
    m_e_max = std::vector<double>(m_e_max.begin() + low_cut, m_e_max.begin() + high_cut);
    m_counts = std::vector<double>(m_counts.begin() + low_cut, m_counts.begin() + high_cut);
  }

  const std::vector<double> & Spectrum::minEnergy() const { return m_e_min; }

  const std::vector<double> & Spectrum::maxEnergy() const { return m_e_max; }

  const std::vector<double> & Spectrum::counts() const { return m_counts; }

  void Spectrum::load(const InputTable & spectrum_table) {
    // Check for existence of columns E_MIN and/or E_MAX, right in the spectrum table.
    bool spec_has_energy = false;
    try {
      spectrum_table.getFieldDim("e_min");
      spec_has_energy = true;
    } catch (const std::exception &) {}
    if (!spec_has_energy) {
      try {
        spectrum_table.getFieldDim("e_max");
        spec_has_energy = true;
      } catch (const std::exception &) {}
    }

    // If spectrum has energy column(s), create Spectrum using spectrum table for ebounds.
    if (spec_has_energy) {
      load(spectrum_table, spectrum_table);
      return;
    }

    // Next try getting EBOUNDS extension from the same file.
    try {
      // Reconstruct full file name from spectrum file name.
      std::string ebounds_file_name = spectrum_table.getDirectoryName();
      if (ebounds_file_name.empty()) ebounds_file_name = spectrum_table.getFileName();
      else ebounds_file_name += "/" + spectrum_table.getFileName();

      std::auto_ptr<InputTable> ebounds_table(new InputTable(ebounds_file_name, "EBOUNDS"));
      load(spectrum_table, *ebounds_table);
      return;
    } catch (const std::exception &) {}

    // Next try reading RESPFILE keyword and looking for that file.
    std::string ebounds_file_name;
    try {
      ebounds_file_name = spectrum_table.getKeyword("RESPFILE");
    } catch (const std::exception &) {}

    // If this is a vector-valued spectrum, see if there is a RESPFILE column.
    if (ebounds_file_name.empty() && 0 <= m_spec_index && spectrum_table.getNumRecords() > (unsigned long) m_spec_index) {
      try {
        spectrum_table.getRecord(m_spec_index).getField("RESPFILE", ebounds_file_name);
      } catch (const std::exception &) {}
    }

    if (!ebounds_file_name.empty()) {
      try {
        std::auto_ptr<InputTable> ebounds_table(new InputTable(ebounds_file_name, "EBOUNDS"));
        load(spectrum_table, *ebounds_table);
        return;
      } catch (const std::exception &) {}
      try {
        // Try looking in the same directory where the spectrum was found.
        std::string ebounds_dir_name = spectrum_table.getDirectoryName();
        if (!ebounds_dir_name.empty()) {
          ebounds_file_name = ebounds_dir_name + "/" + ebounds_file_name;
          std::auto_ptr<InputTable> ebounds_table(new InputTable(ebounds_file_name, "EBOUNDS"));
          load(spectrum_table, *ebounds_table);
          return;
        }
      } catch (const std::exception &) {}
    }

    throw std::runtime_error("Spectrum::load: Could not find EBOUNDS extension for file " + spectrum_table.getFileName());
  }

  void Spectrum::load(const InputTable & spectrum_table, const InputTable & ebounds_table) {
    if (0 == spectrum_table.getNumRecords()) {
      throw std::runtime_error("Spectrum is empty");
    } else if (1 != spectrum_table.getFieldDim("counts")) {
      if (0 > m_spec_index) {
        throw std::logic_error("Spectrum::load: Attempt to open vector-valued spectrum file as a scalar-valued spectrum");
      } else if (spectrum_table.getFieldDim("counts") != ebounds_table.getNumRecords()) {
        throw std::logic_error("Spectrum::load: In vector spectrum, counts column width must agree with number of rows in ebounds");
      }
    } else if (spectrum_table.getNumRecords() != ebounds_table.getNumRecords()) {
      if (0 <= m_spec_index) {
        throw std::logic_error("Spectrum::load: Attempt to open scalar-valued spectrum file as a vector-valued spectrum");
      } else if (spectrum_table.getNumRecords() != ebounds_table.getNumRecords()) {
        throw std::logic_error("Spectrum::load: In scalar spectrum, counts and ebounds columns must have same number of rows");
      }
    }

    if (0 > m_spec_index)
      // Counts are from scalar column in spectrum table.
      spectrum_table.getColumn("counts", m_counts);
    else
      // Counts are from vector column in spectrum table.
      spectrum_table.getRecord(m_spec_index).getField("counts", m_counts);

    // Get energy bounds from ebounds table.
    loadEnergyBounds(ebounds_table);
  }

  std::string Spectrum::getFileName() const { return m_spec_file->getFileName(); }

  const SpecFile * Spectrum::getSpecFile() const { return m_spec_file; }

  void Spectrum::loadEnergyBounds(const InputTable & ebounds_table) {
    // Minimum energy bounds must be present in ebounds table.
    ebounds_table.getColumn("e_min", m_e_min);

    // Put counts into a map indexed on e_min, so that the order can be restored after e_min is sorted.
    std::map<double, double> count_map;
    for (std::vector<double>::size_type idx = 0; idx < m_counts.size(); ++idx) {
      count_map.insert(std::make_pair(m_e_min[idx], m_counts[idx]));
    }

    try {
      // Maximum energy bounds may or may not be present.
      ebounds_table.getColumn("e_max", m_e_max);

      // Make sure each interval individually has width, and that min < max.
      for (std::vector<double>::size_type idx = 0; idx != m_e_max.size(); ++idx) {
        if (m_e_min[idx] >= m_e_max[idx])
          throw std::runtime_error("E_MIN must be < E_MAX for all rows of spectrum file");
      }
    } catch (const std::exception &) {
      // TODO: write a warning about e_max not being found and discarding the last point in the spectrum.
    }

    // Sort minimum energies into ascending order.
    std::sort(m_e_min.begin(), m_e_min.end());

    // Restore counts to correct order.
    for (std::vector<double>::size_type idx = 0; idx < m_counts.size(); ++idx) {
      m_counts[idx] = count_map[m_e_min[idx]];
    }

    if (m_e_max.empty()) {
      // No e_max field: use e_min from next row for each e_max.
      if (m_e_min.size() > 1) {
        m_e_max.assign(m_e_min.begin() + 1, m_e_min.end());
        m_e_min.resize(m_e_min.size() - 1);
        m_counts.resize(m_counts.size() - 1);
      } else {
        throw std::runtime_error("If E_MAX field is not present, E_MIN must contain at least 2 bins");
      }
    } else {
      // Maximum energies were found in file, so sort them into ascending order also.
      std::sort(m_e_max.begin(), m_e_max.end());
    }

    // Make sure no two intervals start or end with same value.
    for (std::vector<double>::size_type idx = 1; idx < m_e_max.size(); ++idx) {
      if (m_e_min[idx - 1] == m_e_min[idx])
        throw std::runtime_error("E_MIN values must be unique");
      if (m_e_max[idx - 1] == m_e_max[idx])
        throw std::runtime_error("E_MAX values must be unique");
    }

  }

}
