/** \file TestPlasmaDb.cxx
    \brief Implementation of PlasmaDb class-specific parts of the unit test.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <cstdio>
#include <stdexcept>

#include "profit/PlasmaDb.h"
#include "profit/UnitTest.h"

namespace profit {

  void UnitTest::testPlasmaDb() {
    std::string method("testPlasmaDb()");

    // Get plasma database accessor.
    PlasmaDb & database = PlasmaDb::instance();

    // Look for a level with missing Z.
    info(method) << "Looking for level which is not in the plasma database, with Z = 3." << std::endl;
    try {
      database.findLevel(3, 3, 1);
      error(method) << "database.findLevel(3, 3, 1) did not throw an exception." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Look for a level with negative Z.
    info(method) << "Look for level with invalid Z = -1." << std::endl;
    try {
      database.findLevel(-1, -1, 1);
      error(method) << "database.findLevel(-1, -1, 1) did not throw an exception." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Look for a level with missing rmJ.
    info(method) << "Look for level with invalid rmJ, Z = 28, rmJ = 29." << std::endl;
    try {
      database.findLevel(28, 29, 1);
      error(method) << "database.findLevel(28, 29, 1) did not throw an exception." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Look for a level with negative rmJ.
    info(method) << "Looking for level with Z = 28, illegal rmJ = -1." << std::endl;
    try {
      database.findLevel(28, -1, 1);
      error(method) << "database.findLevel(28, -1, 1) did not throw an exception." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Create an undefined atomic level.
    info(method) << "Default-constructing an (undefined) AtomicLevel object." << std::endl;
    AtomicLevel level;
    testAtomicLevel("After AtomicLevel level", level, 0., 0, 0, 0., 0, "");
    info(method) << std::endl;

    // Look for a very specific valid atomic level.
    info(method) << "Finding information in plasma database about Ni XXVIII, level 22." << std::endl;
    level = database.findLevel(28, 28, 22);
    testAtomicLevel("After level = database.findLevel(28, 28, 22)", level, 1.0240300E+04, 5, 3, 2., 6, "5f~^2F_{5/2}");
    info(method) << std::endl;

    info(method) << "Confirming that a dielectronic level returns an \"undefined\" level object." << std::endl;
    level = database.findLevel(1, 1, 10000);
    std::string description = "10000    not available                  -1 -1 -1  -1  0";
    if (description != level.description())
      error(method) << "After level = database.findLevel(1, 1, 10000), level.description() returned \n\"" << level.description() <<
        "\", not \n\"" << description << "\", as expected." << std::endl;
    info(method) << std::endl;

    // Look for an invalid transition.
    info(method) << "Looking for an invalid transition (Z = 28, rmJ = 28, lower_level = 10, upper_level = 10)." << std::endl;
    try {
      database.findTransition(28, 28, 10, 10);
      error(method) << "database.findTransition(28, 28, 10, 10) did not throw an exception." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // TODO: test for a transition which will not be found. Does this just return a valid transition object
    // which has no processes?

    // Test default-constructing an (undefined) transition.
    info(method) << "Default-constructing an (undefined) AtomicTransition object." << std::endl;
    AtomicTransition transition;
    testAtomicTransition("After AtomicTransition transition", transition, AtomicLevel(), AtomicLevel(), 0, 0, 0);
    info(method) << std::endl;

    // Find a radiative transition.
    // TODO: RKS Confirm this is supposed to be radiative only.
    info(method) << "Looking for a valid radiative transition." << std::endl;
    transition = database.findTransition(28, 28, 10, 18);
    testAtomicTransition("After database.findTransition(28, 28, 10, 18)", transition, database.findLevel(28, 28, 10),
      database.findLevel(28, 28, 18), 0, 1, 0);
    info(method) << std::endl;

    // TODO: add tests for collisional and satellite, preferably at least one case where multiple processes
    // and/or processes of different types.

    // Test for bug in which dielectronic processes were not correctly referring to the lower level with
    // the lower rmJ value.
    info(method) << "Looking for dielectronic transition O VIII 10128->41." << std::endl;
    transition = database.findTransition(8, 8, 41, 10128);
    info(method) << std::endl;
  }

  void UnitTest::testAtomicLevel(const std::string & hint, const AtomicLevel & level, double energy, int quantumN, int quantumL,
    double quantumS, int degeneracy, const std::string & config) const {
    std::string method("testAtomicLevel(hint, ...)");

    // Verify the information associated with this level.
    if (!isEqual(energy, level.energy(), 0., s_float_epsilon)) {
      error(method) << hint << ", level.energy() returned " << level.energy() << ", not " <<
        energy << " as expected." << std::endl;
    }

    if (quantumN != level.quantumN()) {
      error(method) << hint << ", level.quantumN() returned " << level.quantumN() << ", not " <<
        quantumN << " as expected." << std::endl;
    }

    if (quantumL != level.quantumL()) {
      error(method) << hint << ", level.quantumL() returned " << level.quantumL() << ", not " <<
        quantumL << " as expected." << std::endl;
    }

    if (!isEqual(quantumS, level.quantumS())) {
      error(method) << hint << ", level.quantumS() returned " << level.quantumS() << ", not " <<
        quantumS << " as expected." << std::endl;
    }

    if (degeneracy != level.degeneracy()) {
      error(method) << hint << ", level.degeneracy() returned " << level.degeneracy() << ", not " <<
        degeneracy << " as expected." << std::endl;
    }

    if (config != level.config()) {
      error(method) << hint << ", level.config() returned \"" << level.config() << "\", not \"" <<
        config << "\" as expected." << std::endl;
    }

  }

  void UnitTest::testAtomicTransition(const std::string & hint, const AtomicTransition & transition,
    const AtomicLevel & lower_level, const AtomicLevel & upper_level, size_t num_coll, size_t num_rad, size_t num_sat) const {
    std::string method("testAtomicTransition()");

    // Confirm lower level matches.
    testAtomicLevel(hint + ", comparing lower_level", transition.getLowerLevel(), lower_level.energy(),
      lower_level.quantumN(), lower_level.quantumL(), lower_level.quantumS(), lower_level.degeneracy(), lower_level.config());

    // Confirm upper level matches.
    testAtomicLevel(hint + ", comparing upper_level", transition.getUpperLevel(), upper_level.energy(),
      upper_level.quantumN(), upper_level.quantumL(), upper_level.quantumS(), upper_level.degeneracy(), upper_level.config());

    // Confirm number of each type of transition.
    if (num_coll != transition.getCollision().size()) {
      error(method) << hint << ", transition.getCollision().size() returned " << transition.getCollision().size() << ", not " <<
        num_coll << " as expected." << std::endl;
    }
    if (num_rad != transition.getRadiative().size()) {
      error(method) << hint << ", transition.getRadiative().size() returned " << transition.getRadiative().size() << ", not " <<
        num_rad << " as expected." << std::endl;
    }
    if (num_sat != transition.getSatellite().size()) {
      error(method) << hint << ", transition.getSatellite().size() returned " << transition.getSatellite().size() << ", not " <<
        num_sat << " as expected." << std::endl;
    }
  }

}
