/** \file TestTable.cxx
    \brief Implementation of Table class-specific parts of the unit test.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <cstdio>
#include <iostream>
#include <memory>
#include <stdexcept>
#include <vector>

#include <sys/stat.h>

#include "profit/Table.h"
#include "profit/UnitTest.h"

namespace profit {

  void UnitTest::testTable() {
    std::string method("testTable()");

    // Attempt to open a Table which refers to a non-existent file.
    info(method) << "Attempting to open table \"PEAKEMIS\" in file \"non-existent-file.fits\"." << std::endl;
    try {
      std::auto_ptr<InputTable> table(new InputTable("non-existent-file.fits", "PEAKEMIS"));
      error(method) << "No exception was thrown when opening non-existent file." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Attempt to open a Table which refers to a non-existent table.
    info(method) << "Attempting to open table \"NON_EXIS\" in file \"" << m_atom_db_file_name << "\"." << std::endl;
    try {
      std::auto_ptr<InputTable> table(new InputTable(m_atom_db_file_name, "NON_EXIS"));
      error(method) << "No exception was thrown when opening non-existent table in existing file." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Attempt to open an image as a table.
    info(method) << "Attempting to open primary image extension as a table in file \"" << m_atom_db_file_name << "\"." << std::endl;
    try {
      std::auto_ptr<InputTable> table(new InputTable(m_atom_db_file_name, "0"));
      error(method) << "No exception was thrown when opening a primary image as a table." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Open PEAKEMIS table.
    info(method) << "Opening table \"PEAKEMIS\" in file \"" << m_atom_db_file_name << "\"." << std::endl;
    std::auto_ptr<InputTable> table(new InputTable(m_atom_db_file_name, "PEAKEMIS"));
    info(method) << std::endl;

    // Confirm number of rows.
    info(method) << "Confirming number of records/rows is correct in table \"PEAKEMIS\"." << std::endl;
    size_t expected_num_rec = 34785;
    if (expected_num_rec != table->getNumRecords()) {
      error(method) << "Expected " << expected_num_rec << ", but found " << table->getNumRecords() <<
        " records in table \"PEAKEMIS\"." << std::endl;
    }
    info(method) << std::endl;

    // Attempt to get a column which doesn't exist.
    info(method) << "Attempting to get column \"NON_EXISTENT\" in file \"" << m_atom_db_file_name << "\"." << std::endl;
    try {
      std::vector<double> column;
      table->getColumn("NON_EXISTENT", column);
      error(method) << "No exception was thrown when getting a non-existent column in table \"PEAKEMIS\"." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Attempt to get a column which does exist.
    info(method) << "Attempting to get column \"LAMBDA\" which should exist in table \"PEAKEMIS\"." << std::endl;
    std::vector<double> lambda;
    table->getColumn("LaMbDa", lambda);
    info(method) << std::endl;

    // Make sure the right number of rows was obtained.
    info(method) << "Confirming size of column \"LAMBDA\" is correct in table \"PEAKEMIS\"." << std::endl;
    if (expected_num_rec != lambda.size())
      error(method) << "Number of values in column \"LAMBDA\" was " << lambda.size() << ", not " << expected_num_rec <<
        ", as expected." << std::endl;
    info(method) << std::endl;

    // Spot-check first, last and center values. These are in a fits column of format 1E, hence they can only
    // be expected to agree as well as a float would.
    info(method) << "Attempting to access field \"LAMBDA \" using column-oriented accessors" << std::endl;
    size_t expected_index[] = { 0, 17392, 34784 };
    double expected_lambda[] = { 1.1982250E+00, 4.8695000E+01, 1.8207626E+07 };
    for (size_t index = 0; index < sizeof(expected_index) / sizeof(size_t); ++index) {
      if (!isEqual(lambda[expected_index[index]], expected_lambda[index], 0., s_float_epsilon)) {
        error(method) << "Value of column \"LAMBDA\" in table \"PEAKEMIS\", row " << expected_index[index] + 1 << " was " <<
          lambda[expected_index[index]] << ", not " << expected_lambda[index] << " as expected." << std::endl;
      }
    }
    info(method) << std::endl;

    // Test getting the name of the file from the table object.
    info(method) << "Confirming Table::getFileName returns the correct file name." << std::endl;
    if (m_atom_db_file_name != table->getFileName())
      error(method) << "Table::getFileName() returned the name \"" << table->getFileName() <<
        "\", not \"" << m_atom_db_file_name << "\" as expected." << std::endl;
    info(method) << std::endl;

    // Test getting a keyword which is not actually present.
    info(method) << "Attempting to get keyword \"NON_EXIS\" in table \"PEAKEMIS\"." << std::endl;
    try {
      table->getKeyword("NON_EXIS");
      error(method) << "No exception was thrown when getting a non-existent keyword from table \"PEAKEMIS\"." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Test getting a keyword which is present.
    info(method) << "Confirming keyword \"EXTNAME\" is correct in table \"PEAKEMIS\"." << std::endl;
    std::string expected_ext_name = "PEAKEMIS";
    std::string ext_name = table->getKeyword("EXTNAME");
    if (expected_ext_name != ext_name)
      error(method) << "In table \"PEAKEMIS\", keyword \"EXTNAME\" was \"" << ext_name <<
        "\", not \"" << expected_ext_name << "\", as expected." << std::endl;
    info(method) << std::endl;

    // Test record/row-oriented access.
    // Attempt to access a record which is not present.
    try {
      Record::size_type index = 34785;
      info(method) << "Attempting to get record number " << index << ", which should not exist in table \"PEAKEMIS\"." << std::endl;
      table->getRecord(index);
      error(method) << "No exception was thrown when getting invalid record from table \"PEAKEMIS\"." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Get a record which is present, but attempt to get a field from that record which does not exist.
    info(method) << "Attempting to get field \"NON_EXIS\" from record 0 in table \"PEAKEMIS\"." << std::endl;
    Record record = table->getRecord(0);
    try {
      record.getField("NON_EXIS");
      error(method) << "No exception was thrown when getting non-existent field from table \"PEAKEMIS\"." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Spot-check first, last and center values, accessing them row-wise rather than column-wise. These are
    // in a fits column of format 1E, hence they can only be expected to agree as well as a float would.
    info(method) << "Attempting to access field \"LAMBDA\" using record/row-oriented accessors" << std::endl;
    for (size_t index = 0; index < sizeof(expected_index) / sizeof(size_t); ++index) {
      double lambda_scalar = table->getRecord(expected_index[index]).getField("Lambda");
      if (!isEqual(lambda_scalar, expected_lambda[index], 0., s_float_epsilon)) {
        error(method) << "In \"PEAKEMIS\" table, row " << expected_index[index] + 1 << ", value of lambda column was " <<
          lambda_scalar << ", not " << expected_lambda[index] << " as expected." << std::endl;
      }
    }
    info(method) << std::endl;

    // Test new filtering syntax.
    info(method) << "Attempting to filter table \"PEAKEMIS\"." << std::endl;
    table->filter("element==28&&ion==28");
    if (table->getNumRecords() != 16)
      error(method) << "After table->filter(\"element==28&&ion==28\"), table \"PEAKEMIS\" has " << table->getNumRecords() <<
        ", not 16 as expected." << std::endl;
    info(method) << std::endl;

    // Test opening a read-only file.
    info(method) << "Changing mode of test spectrum file to be read-only." << std::endl;
    chmod(m_spectrum_file_name.c_str(), S_IRUSR);
    info(method) << std::endl;

    info(method) << "Re-opening " << m_spectrum_file_name << " now that it is read-only on disk." << std::endl;
    try {
      table.reset(new InputTable(m_spectrum_file_name, "SPECTRUM"));
    } catch (const std::exception & x) {
      error(method) << "Re-opening a table which is read-only on disk threw exception: " << x.what() << std::endl;
    }
    info(method) << std::endl;

    info(method) << "Changing mode of file back to read-write." << std::endl;
    chmod(m_spectrum_file_name.c_str(), S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
    info(method) << std::endl;

    // Test reading a text file.
    info(method) << "Attempting to open a text file." << std::endl;
    std::auto_ptr<InputTable> text_table(new InputTable(m_text_spectrum_file_name, ""));
    expected_num_rec = 13;
    if (expected_num_rec != text_table->getNumRecords())
      error(method) << "In text file, number of records was " << text_table->getNumRecords() << ", not " <<
        expected_num_rec << ", as expected." << std::endl;
    info(method) << std::endl;

    // Try to get a non-existent keyword from the text file.
    info(method) << "Attempting to find a keyword which is not present from a text file." << std::endl;
    try {
      table->getKeyword("!NOT_A_KEYWORD");
      error(method) << "No exception was thrown when getting a non-existent keyword from text file." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }

    // Try to get an existing keyword from the text file.
    info(method) << "Attempting to find a keyword which does exist from a text file." << std::endl;
    std::string expected_object = "Crab";
    std::string object = text_table->getKeyword("obJEct");
    if (expected_object != object)
      error(method) << "From a text file, getKeyword(\"obJEct\") returned \"" << object <<
        "\", not \"" << expected_object << "\", as expected." << std::endl;
    info(method) << std::endl;

    // Try to get a non-existent column from the text file.
    info(method) << "Attempting to find a column which is not present from a text file." << std::endl;
    try {
      std::vector<double> column;
      text_table->getColumn("NOT_A_FIELD", column);
      error(method) << "No exception was thrown when attempting to access a column which does not exist in text file." << std::endl;
    } catch (const std::exception & x) {
      info(method) << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Try to get an existing column from the text file.
    info(method) << "Attempting to find a column which does exist in a text file." << std::endl;
    try {
      std::vector<double> column;
      text_table->getColumn("couNTS", column);
      double expected_column[] = { 0., 1., 3., 1., 4., 6., 2., 5., 8., 3.,  2.,  1.,  0. };
      info(method) << "Verifying size of counts column." << std::endl;
      if (column.size() != sizeof(expected_column) / sizeof(double)) {
        error(method) << "Column \"couNTS\" has size " << column.size() << ", not " << sizeof(expected_column) / sizeof(double) <<
          ", as expected." << std::endl;
      } else {
        info(method) << std::endl;
        info(method) << "Verifying content of counts column." << std::endl;
        for (Record::size_type index = 0; index != column.size(); ++index) {
          if (column[index] != expected_column[index])
            error(method) << "Column couNTS[" << index << "] is " <<
            column[index] << ", not " << expected_column[index] << ", as expected." << std::endl;
        }
      }
  
    } catch (const std::exception & x) {
      error(method) << "Attempting to find column \"couNTS\", unexpectedly threw exception: " << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Try to get an existing column from a filtered text file.
    info(method) << "Attempting to filter a column which does exist in a text file." << std::endl;
    try {
      // Apply filter.
      text_table->filter("counts", 3., 6.);

      std::vector<double> column;
      text_table->getColumn("couNTS", column);

      double expected_column[] = { 3., 4., 6., 5., 3. };
      info(method) << "Verifying size of counts column." << std::endl;
      if (column.size() != sizeof(expected_column) / sizeof(double)) {
        error(method) << "Column \"couNTS\" has size " << column.size() << ", not " << sizeof(expected_column) / sizeof(double) <<
          ", as expected." << std::endl;
      } else {
        info(method) << std::endl;
        info(method) << "Verifying content of counts column." << std::endl;
        for (Record::size_type index = 0; index != column.size(); ++index) {
          if (column[index] != expected_column[index])
            error(method) << "Column couNTS[" << index << "] is " <<
            column[index] << ", not " << expected_column[index] << ", as expected." << std::endl;
        }
      }
  
    } catch (const std::exception & x) {
      error(method) << "Attempting to find column \"couNTS\", unexpectedly threw exception: " << x.what() << std::endl;
    }
    info(method) << std::endl;

    // Test reading an unformatted text file.
    info(method) << "Attempting to open an unformatted text file." << std::endl;
    try {
      std::auto_ptr<InputTable> unfmt_table(new InputTable(m_unfmt_file_name, ""));
      error(method) << "Attempting to open unformatted text file did not throw exception." << std::endl;
    } catch (const std::exception & x) {
      info(method) << "Attempting to open unformatted text file threw exception: " << x.what() << std::endl;
    }
    info(method) << std::endl;
  }

}
