/** \file UnitTest.cxx
    \brief Implementation of class UnitTest main features.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <cmath>
#include <cstdlib>
#include <fstream>
#include <limits>
#include <sstream>
#include <stdexcept>
#include <unistd.h>

#include "profit/UnitTest.h"
#include "profit/UnitTestGui.h"

namespace profit {

  // Value of epsilon for doubles, based on machine epsilon but allowing for expected degradation due to repeated
  // floating point operations.
  double UnitTest::s_double_epsilon = 5. * std::numeric_limits<double>::epsilon();

  // Value of epsilon for floats, based on machine epsilon but allowing for expected degradation due to repeated
  // floating point operations.
  double UnitTest::s_float_epsilon = 5. * std::numeric_limits<float>::epsilon();

  UnitTest::UnitTest(): m_atom_db_env(), m_atom_db_file_name(), m_xstar_db_file_name(),
    m_spectrum_file_name("xrs.fak"), m_ebounds_file_name("xrs.rmf"), m_text_spectrum_file_name("xrs.txt"),
    m_unfmt_file_name("xstar.dat"), m_unordered_text_file_name("xrs-mixed.txt") {
    std::string method("UnitTest::UnitTest()");

    info(method) << "Checking runtime environment." << std::endl;

    // Get ATOMDB environment variable.
    const char * atom_db_cp = getenv("ATOMDB");
    if (0 == atom_db_cp) {
      error(method) << "Environment variable \"ATOMDB\" is not set." << std::endl;
    } else {
      m_atom_db_env = atom_db_cp;
      m_atom_db_file_name = m_atom_db_env + "/apec_linelist.fits";
      if (!fileExists(m_atom_db_file_name)) {
        error(method) << "File \"" << m_atom_db_file_name << "\" does not exist." << std::endl;
      }
    }

    // Get PROFIT environment variable.
    const char * profit_env_var = getenv("PROFIT");
    if (0 == profit_env_var) {
      error(method) << "Environment variable \"PROFIT\" is not set." << std::endl;
    } else {
      m_xstar_db_file_name = std::string(profit_env_var) + "/data/xstardb/xstar_linelist.dat";
      if (!fileExists(m_xstar_db_file_name)) {
        error(method) << "File \"" << m_xstar_db_file_name << "\" does not exist." << std::endl;
      }
    }

// TODO: check PFILES.

    // Check for test spectrum.
    if (!fileExists(m_spectrum_file_name))
      error(method) << "File \"" << m_spectrum_file_name << "\" does not exist." << std::endl;

    // Check for test ebounds.
    if (!fileExists(m_ebounds_file_name))
      error(method) << "File \"" << m_ebounds_file_name << "\" does not exist." << std::endl;

    // Check for test text file.
    if (!fileExists(m_text_spectrum_file_name))
      error(method) << "File \"" << m_text_spectrum_file_name << "\" does not exist." << std::endl;

    // Check for test unformatted text file.
//    if (!fileExists(m_unfmt_file_name))
//      error(method) << "File \"" << m_unfmt_file_name << "\" does not exist." << std::endl;

    // Check for test text file.
    if (!fileExists(m_unordered_text_file_name))
      error(method) << "File \"" << m_unordered_text_file_name << "\" does not exist." << std::endl;

    // Check for xstar db text file.
    if (!fileExists(m_xstar_db_file_name))
      error(method) << "File \"" << m_xstar_db_file_name << "\" does not exist." << std::endl;

    info(method) << std::endl;
  }

  UnitTest::~UnitTest() {}

  void UnitTest::testSpectrumView() {
    std::string method("UnitTest::testSpectrumView()");
    info(method) << "this test not implemented (GUI-related class)" << std::endl;
    info(method) << std::endl;
  }

  void UnitTest::testGui() {
    std::string method("UnitTest::testGui()");
    info(method) << "this test not implemented (GUI-related class)" << std::endl;
    info(method) << std::endl;
  }

  void UnitTest::runTest(void (UnitTest::* test)()) {
    std::string method("runTest(...)");
    if (s_debug_mode) {
      (this->*test)();
    } else {
      try {
        (this->*test)();
      } catch (const std::exception & x) {
        error(method) << x.what() << std::endl;
      }
    }
  }

  void UnitTest::runApp() {
    std::string method("UnitTest::runApp()");
    if (!hitMe()) {
      runTest(&UnitTest::testTable);
      runTest(&UnitTest::testEnergy);
      runTest(&UnitTest::testAtomDb);
      runTest(&UnitTest::testPlasmaDb);
      runTest(&UnitTest::testSpectrum);
      runTest(&UnitTest::testSpecFile);
      runTest(&UnitTest::testSpectrumView);
      runTest(&UnitTest::testStream);
      runTest(&UnitTest::testGui);

      // Handle test failure.
      if (failed()) {
        error(method) << "TEST FAILED" << std::endl;
      } else {
        info(method) << "Test Passed." << std::endl;
      }
    }
  }

  bool UnitTest::fileExists(const std::string & file_name) const {
    std::ifstream file(file_name.c_str());
    return file.good();
  }

  bool UnitTest::isEqual(double val1, double val2, double abs_rel_epsilon, double rel_epsilon) const {
    return isAbsEqual(val1, val2, abs_rel_epsilon) ? true : isRelEqual(val1, val2, rel_epsilon);
  }

  bool UnitTest::isAbsEqual(double val1, double val2, double epsilon) const {
    return val1 == val2 ? true : epsilon > fabs(val1 - val2);
  }

  bool UnitTest::isRelEqual(double val1, double val2, double epsilon) const {
    bool equal = val1 == val2;
    if (!equal) {
      // val1 and val2 are not exactly equal.
      // Guess that |val2| > |val1| so that |val2 / val1| > 1. Protect against division by 0.
      double quotient = (0. != val1) ? val2 / val1 : 0.;

      // quotient < 0. implies val1 and val2 have different signs, so they are not approximately equal
      // in a relative sense.
      //
      // quotient == 0. implies val1 == 0. or val2 == 0. but not both because that was ruled out
      // above, so they are not approximately equal in a relative sense.
      //
      // 0. < quotient < 1. implies that the guess above was wrong, that is |val1| > |val2| > 0., but at
      // least val1 and val2 have same sign, so compute the reciprocal of the quotient which was computed above.
      if (0. < quotient && quotient < 1.) quotient = val1 / val2;

      // At this point, 1. < quotient < 1. + epsilon implies that one of the two ways of computing the
      // quotient produced a comparison within the valid range, hence approximate equality.
      equal = 1. < quotient && quotient < 1. + epsilon;
    }
    return equal;
  }

  bool UnitTest::hitMe() {
    std::string method("UnitTest::hitMe()");
    bool hit = false;

    if (s_hit_mode) {
      // Set up a "hit" on this process in case it hangs.
      pid_t orig_pid = getpid();
      pid_t pid = fork();

      if (0 == pid) {
        std::ostringstream os;
        os << "kill -9 " << orig_pid;

        // Child, whose job is to terminate the parent.
        sleep(60);
        error(method) << "EXECUTING " << os.str() << std::endl;
        system(os.str().c_str());
        hit = true;
      }
    }

    return hit;
  }

}
