/** \file UnitTestGui.cxx
    \brief Implementation of UnitTestGui class main functions.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <stdexcept>

#include "TApplication.h"
#include "TGButton.h"
#include "TGClient.h"
#include "TGFrame.h"

#include "profit/Gui.h"
#include "profit/SpectrumView.h"
#include "profit/UnitTestGui.h"

ClassImp(profit::UnitTestGui)

namespace {

  class UTMainFrame : public TGMainFrame {
    public:
      UTMainFrame();

      virtual void CloseWindow();
  };

  UTMainFrame::UTMainFrame(): TGMainFrame(gClient->GetRoot(), 675, 600) {
    SetWindowName("profit graphical unit test");
  }

  void UTMainFrame::CloseWindow() { gApplication->Terminate(0); }

}

namespace profit {

  UnitTestGui::UnitTestGui(): UnitTest(), m_frame_cont(), m_temp_frame_cont(), m_main_frame(0), m_spectrum_view(0),
    m_test_ran(false) {
    // Perform Root global initializations.
    Gui::initRoot();

    // Create the GUI.
    try {
      // Create main frame.
      m_main_frame = new UTMainFrame;
      storeFrame("unit test main frame", m_main_frame);

      // Create "run test" button.
      TGTextButton * run_test = new TGTextButton(m_main_frame, "Run Unit Test");
      storeFrame("Run Unit Test Button", run_test);
      m_main_frame->AddFrame(run_test);
      run_test->Connect("Clicked()", "profit::UnitTestGui", this, "runAllTests()");

    } catch (...) {
      clear();
      throw;
    }

    // Display GUI.
    m_main_frame->MapSubwindows();
    m_main_frame->Layout();
    m_main_frame->MapWindow();
  }

  UnitTestGui::~UnitTestGui() {
    clear();
  }

  void UnitTestGui::runAllTests() {
    m_test_ran = false;
    UnitTest::runApp();
    //testSpectrumView();
    //testGui();
    //testStream();
    m_test_ran = true;
  }

  void UnitTestGui::runApp() {
    // Run application/event loop.
    gApplication->Run(kTRUE);
    if (!m_test_ran) error("UnitTestGui::run()") << "Unit test was not run prior to exit." << std::endl;
  }

  void UnitTestGui::storeFrame(const std::string & context, TGFrame * frame) {
    storeFrame(context, frame, m_frame_cont);
  }

  void UnitTestGui::storeFrame(const std::string & context, TGFrame * frame, FrameCont & cont) {
    if (0 == frame) throw std::runtime_error("Allocation of " + context + " failed");
    cont.push_back(frame);
  }

  void UnitTestGui::clear() {
    delete m_spectrum_view;
    clearFrames(m_temp_frame_cont);
    clearFrames(m_frame_cont);
  }

  void UnitTestGui::clearFrames(FrameCont & cont) {
    // Delete all widgets in reverse order of creation.
    while (!cont.empty()) {
      // First get the pointer in question.
      TGFrame * frame = cont.back();
      // Detach each frame from its parent.
      TGCompositeFrame * parent = const_cast<TGCompositeFrame *>(dynamic_cast<const TGCompositeFrame *>(frame->GetParent()));
      if (0 != parent) parent->RemoveFrame(frame);
      // Remove this pointer from the container of all frames.
      cont.pop_back();
      // Destroy the frame.
      delete frame;
    }
  }

}
